/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.api;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.speedo.genclass.api.SpeedoGenClassCoherence;
import org.objectweb.speedo.genclass.api.SpeedoGenClassListener;
import org.objectweb.speedo.mim.api.SpeedoProxy;

/**
 * This interfaces represents a SpeedoProxy for a generic class (Collectio, Set,
 * Map, ...)
 * It permits to specify the mapper throught the generic class is persistent.
 * The Mapper name permits to know the type of GenClassMapping and
 * GenClassBinding (rdb, fos, ...).
 *
 * The PType of the generic class permit to known which is GenClassMapping
 * instance to use.
 *
 * @author S.Chassande-Barrioz
 */
public interface SpeedoGenClassProxy extends SpeedoProxy, SpeedoGenClassCoherence {

    /**
     * Assignes the PType of the generic class.
     * @param ptype of the generic class
     */
    void jdoSetPType(PType ptype);

    /**
     * @return the PType of the generic class.
     */
    PType jdoGetPType();

    /**
     * @return a string which represents the fully path of the persistent field
     * of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the returned string will be 'com.foo.Bar/f1'
     */
    String jdoGetGenClassId();

    /**
     * Assignes the linked field name for this generic class.
     * @param lf is tring which represents the fully path of the persistent
     * field of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the expected string is 'com.foo.Bar/f1'
     */
    void jdoSetLinkedField(String lf);

    /**
     * assignes the PBinding to use to load/store the generic class
     * Call this method with null value means that the generic is no more
     * persistent.
     * @param pb is the PBinding.
     */
    void jdoSetPBinding(PBinding pb);

    /**
     * @return the pbinding associated to the generic class. The value is not
     * used if the generic class is not persistent.
     */
    PBinding jdoGetPBinding();

    /**
     * Assignes the new value of the generic class (Collection, Map or Array
     * or ...).
     */
    void setElements(Object o);

	Object createGenClass();

    /**
     * Sets the id of this gen class instance (id of the field that references
     * this instance).
     *
     * @param gcid the new id of this gen class instance.
     */
    void speedoSetGcId (int gcid);

    /**
     * Adds a listener to this gen class instance. This listener will be
     * notified each time an element is added or removed for this gen class.
     *
     * @param l a gen class listener.
     */
    void speedoAddGenClassListener (SpeedoGenClassListener l);

    /**
     * Notifies the listeners of this gen class that an element has been added
     * to this gen class.
     *
     * @param elem the element that has been added.
     */
    void fireSpeedoElementAdded (Object elem);

    /**
     * Notifies the listeners of this gen class that an element has been removed
     * to this gen class.
     *
     * @param elem the element that has been removed.
     */
    void fireSpeedoElementRemoved (Object elem);

    void clear ();

	/**
	 * Initializes a Genclass instance
	 * @param ptype is the PType of the genclass
	 * @param linkedField is the name of the field referencing the genclass
	 * @param pnameHints is a helper using for the identifier allocation (export)
	 * persistent
	 * @param logger is the logger instance for logging.
	 */
	void init(PType ptype,
			  String linkedField,
			  Object pnameHints,
			  Logger logger);

	/**
	 * Assignes the hints needed to build the PName of the persistent object
	 * during the export action.
	 */
	void jdoSetPNameHints(Object hints);
}
