/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class HashSetImpl
        extends HashSet
        implements SpeedoGenClassProxy {

    public HashSetImpl() {
        super();
		accessor = (SetAccessor) createAccessor();
    }

	SetAccessor accessor;

	public SpeedoAccessor getReferenceAccessor() {
		return accessor;
	}
	public void setReferenceAccessor(SpeedoAccessor refAcc) {
		accessor = (SetAccessor) refAcc;
	}


    // IMPLEMENTS THE HashSet CLASS //
    //------------------------------//

    public boolean add(Object o) {
        boolean result;
        if (!jdoIsActive()) {
            result = accessor.add(o);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().writeIntention(this, null);
            result = ca.add(o);
        }
        return result;
    }

    public boolean addAll(Collection c) {
        boolean result;
        if (!jdoIsActive()) {
            result = accessor.addAll(c);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().writeIntention(this, null);
            result = ca.addAll(c);
        }
        return result;
    }

    public void clear() {
        if (!jdoIsActive()) {
            accessor.clear();
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().writeIntention(this, null);
            ca.clear();
        }
    }

    public boolean contains(Object o) {
        if (!jdoIsActive()) {
            return accessor.contains(o);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.contains(o);
        }
    }

    public boolean containsAll(Collection c) {
        if (!jdoIsActive()) {
            return accessor.containsAll(c);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.containsAll(c);
        }
    }

    public boolean equals(Object o) {
        if (!jdoIsActive()) {
            return accessor.equals(o);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.equals(o);
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive()) {
            return accessor.isEmpty();
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.isEmpty();
        }
    }

    public Iterator iterator() {
        if (!jdoIsActive()) {
            return accessor.iterator();
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.iterator();
        }
    }

    public boolean remove(Object o) {
        boolean result;
        if (!jdoIsActive()) {
            result = accessor.remove(o);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().writeIntention(this, null);
            result = ca.remove(o);
        }
        return result;
    }

    public boolean removeAll(Collection c) {
        boolean result;
        if (!jdoIsActive()) {
            result = accessor.removeAll(c);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().writeIntention(this, null);
            result = ca.removeAll(c);
        }
        return result;
    }

    public boolean retainAll(Collection c) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        if (!jdoIsActive()) {
            return accessor.size();
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.size();
        }
    }

    public Object[] toArray() {
        if (!jdoIsActive()) {
            return accessor.toArray();
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.toArray();
        }
    }

    public Object[] toArray(Object[] a) {
        if (!jdoIsActive()) {
            return accessor.toArray(a);
        } else {
			SetAccessor ca = (SetAccessor) getSpeedoHome().readIntention(this, null);
            return ca.toArray(a);
        }
    }

    // IMPLEMENTS THE SpeedoGenClassProxy INTERFACE //
    //----------------------------------------------//

    public Object createGenClass() {
        return new HashSet();
    }

    public SpeedoAccessor createAccessor() {
        return new SetAccessor(this);
    }
}
