/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.map;

import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import java.util.Map;
import java.util.Set;
import java.util.Collection;
import java.util.HashMap;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class HashMapImpl
        extends HashMap
        implements SpeedoGenClassProxy {

    public HashMapImpl() {
        super();
		accessor = (MapAccessor) createAccessor();
    }

    public String toString() {
        return "";
    }

	MapAccessor accessor;

	public SpeedoAccessor getReferenceAccessor() {
		return accessor;
	}
	public void setReferenceAccessor(SpeedoAccessor refAcc) {
		accessor = (MapAccessor) refAcc;
	}

    public Object createGenClass() {
        return new HashMap();
    }

    public SpeedoAccessor createAccessor() {
        return new MapAccessor(this);
    }

    // IMPLEMENTS THE HashSet CLASS //
    //------------------------------//

    public int size() {
        if (!jdoIsActive()) {
            return accessor.size();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.size();
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive()) {
            return accessor.isEmpty();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.isEmpty();
        }
    }

    public boolean containsValue(Object o) {
        if (!jdoIsActive()) {
            return accessor.containsValue(o);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.containsValue(o);
        }
    }

    public synchronized boolean containsKey(Object o) {
        if (!jdoIsActive()) {
            return accessor.containsKey(o);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.containsKey(o);
        }
    }

    public synchronized Object get(Object o) {
        if (!jdoIsActive()) {
            return accessor.get(o);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.get(o);
        }
    }

    public synchronized Object put(Object o, Object o1) {
        if (!jdoIsActive()) {
            return accessor.put(o, o1);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            return ma.put(o, o1);
        }
    }

    public synchronized Object remove(Object o) {
        if (!jdoIsActive()) {
            return accessor.remove(o);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            return ma.remove(o);
        }
    }

    public synchronized void putAll(Map map) {
        if (!jdoIsActive()) {
            accessor.putAll(map);
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            ma.putAll(map);
        }
    }

    public synchronized void clear() {
        if (!jdoIsActive()) {
            accessor.clear();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            ma.clear();
        }
    }

    public Set keySet() {
        if (!jdoIsActive()) {
            return accessor.keySet();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.keySet();
        }
    }

    public Set entrySet() {
        if (!jdoIsActive()) {
            return accessor.entrySet();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.entrySet();
        }
    }

    public Collection values() {
        if (!jdoIsActive()) {
            return accessor.values();
        } else {
			MapAccessor ma = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ma.values();
        }
    }
}

