/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz
 *
 */

package org.objectweb.speedo.generation;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;
import org.objectweb.speedo.generation.compiler.Compiler;
import org.objectweb.speedo.generation.enhancer.PersistentClassEnhancer;
import org.objectweb.speedo.generation.enhancer.PersistenceAwareEnhancer;
import org.objectweb.speedo.generation.enhancer.AroundCompilation;
import org.objectweb.speedo.generation.generator.SpeedoGenerator;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.generation.parser.JdoParser;
import org.objectweb.speedo.generation.jdo.SequenceGenerator;
import org.objectweb.speedo.generation.jorm.JormGenerator;
import org.objectweb.speedo.generation.mivisitor.MetaInfoVisitors;
import org.objectweb.speedo.generation.mivisitor.JMICleanerVisitor;
import org.objectweb.speedo.generation.recompiler.UpToDateVisitor;
import org.objectweb.speedo.generation.recompiler.Recompiler;
import org.objectweb.speedo.generation.serializer.MISerializer;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.Monolog;

import java.util.HashMap;

/**
 * TODO: put the file of the meta information into the jar file
 * @author S.Chassande-Barrioz
 */
public class SpeedoCompiler extends AbstractGeneratorComponent {

    public final static String LOGGER_NAME =
            SpeedoProperties.LOGGER_NAME + ".generation.SpeedoCompiler";

    public Object[][] actions = null;

    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//
    public SpeedoCompilerParameter getSpeedoCompilerParameter() {
        if (scp == null)
            scp = new SpeedoCompilerParameter();
        return scp;
    }

    public boolean init() throws SpeedoException {
        // Initialize the logger factory if necessary
        if (scp.loggerFactory == null) {
            if (scp.logPropFile == null) {
                scp.loggerFactory = Monolog.initialize();
            } else {
                scp.loggerFactory = Monolog.getMonologFactory(scp.logPropFile);
            }
            logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        } else if (logger == null) {
            logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        }
        scp.nmf.setLogger(scp.loggerFactory.getLogger(SpeedoProperties.LOGGER_NAME + ".naming"));

        if (actions == null) {
            actions = new Object[][]{
                {new JdoParser(), "Loading jdo files..."},
                {new UpToDateVisitor(), "Computing classes requiring enhancement ..."},
                {new Recompiler(), "Recompiling classes ..."},
                {new MetaInfoVisitors(), "Completing the meta data..."},
                {new JormGenerator(), "Generating Jorm files..."},
				{new SequenceGenerator(), null},
				{new MISerializer(), "Serializing Meta Info..."},
                {new SpeedoGenerator(), "Generating Java files..."},
                {new JMICleanerVisitor(), null},
                {new AroundCompilation(true), null},
                {new Compiler(), "Compiling Java files..."},
                {new AroundCompilation(false), null},
                {new PersistentClassEnhancer(), "Enhancing Persistent classes..."},
                {new PersistenceAwareEnhancer(), "Enhancing Persistent aware classes..."}
            };
        }
        scp.setXmldescriptor(new HashMap());
	    return true;
    }

    public void process() throws SpeedoException {
        if (scp.jdo.isEmpty())
            return;

        // Launch the 5 steps of the speedo compiler
        GeneratorComponent gc = null;
	    boolean run = true;
        for (int i = 0; i < actions.length && run; i++) {
            gc = (GeneratorComponent) actions[i][0];
            gc.setSpeedoCompilerParameter(scp);
            if (gc.init()) {
	            if (actions[i][1] != null) {
	                logger.log(BasicLevel.INFO, actions[i][1]);
	            }
                gc.process();
            } else {
	            run = false;
            }
	        actions[i][0] = null;
	        actions[i][1] = null;
	        System.gc();
        }
        logger.log(BasicLevel.INFO, "Done.");
    }
}
