/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;
import java.util.Iterator;

public class AroundCompilation extends EnhancerComponent {

    public final static String LOGGER_NAME
        = SpeedoProperties.LOGGER_NAME + ".generation.enhancer.precompile";

    boolean beforeCompilation;

    public AroundCompilation(boolean beforeCompilation) {
        this.beforeCompilation = beforeCompilation;
    }

    public boolean init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
		debug = logger.isLoggable(BasicLevel.DEBUG);
	    return !scp.getXmldescriptor().isEmpty();
    }

    public void process() throws SpeedoException {
        if (scp.getXmldescriptor() == null || scp.getXmldescriptor().isEmpty()) {
            return;
        }
        Collection xmls = scp.getXmldescriptor().values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itclass = desc.getSpeedoClasses().iterator(); itclass.hasNext();) {
                SpeedoClass jdoclass = (SpeedoClass) itclass.next();
				if (debug) {
					logger.log(BasicLevel.DEBUG,
							"Arround the compilation of the class "
							+ jdoclass.getFQName() + ": "
							+ (beforeCompilation ? "before" : "after")
							+ " the compilation");
				}
                ClassWriter cw = new ClassWriter(false);
                InheritanceModifier im = new InheritanceModifier(
                    cw, logger, jdoclass, beforeCompilation);
				DummyAccessorModifier dam = new DummyAccessorModifier(
					im, logger, jdoclass, beforeCompilation);
                String name = jdoclass.getFQName();
                ClassReader cr = loadJavaClass(false, name, scp.output, false);
                if (beforeCompilation) {
					EmptyConstructorAdder eca = new EmptyConstructorAdder(
						dam, logger, jdoclass.noArgConstructorStatus);
					PersistentFieldRemover pfr = new PersistentFieldRemover(
							eca, jdoclass, logger);
					cr.accept(pfr, false);
				} else {
					cr.accept(dam, false);
				}
                writeJavaClass(name, cw, scp.output);
            }
        }
    }
}
