/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.enhancer;

import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.asm.Attribute;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Set;

public class DummyAccessorAdder extends EnhancerComponent {

    public final static String LOGGER_NAME
        = SpeedoProperties.LOGGER_NAME + ".generation.enhancer.fieldMdodifier";

    boolean add;

    public DummyAccessorAdder(boolean add) {
        this.add = add;
    }

    public boolean init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
	    return !scp.getXmldescriptor().isEmpty();
    }

    public void process() throws SpeedoException {
        if (scp.getXmldescriptor() == null || scp.getXmldescriptor().isEmpty()) {
            return;
        }
        Collection xmls = scp.getXmldescriptor().values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itclass = desc.getSpeedoClasses().iterator(); itclass.hasNext();) {
                Set fields = new HashSet();
                SpeedoClass jdoclass = (SpeedoClass) itclass.next();
                Iterator it = jdoclass.jdoField.values().iterator();
                while (it.hasNext()) {
                    SpeedoField sf = (SpeedoField) it.next();
                    if (sf.relationType != SpeedoField.NO_RELATION) {
                        fields.add(sf);
                    }
                }
                if (fields.isEmpty()) {
                    continue;
                }
                ClassWriter cw = new ClassWriter(false);
                DummyAccessorModifier fam = new DummyAccessorModifier(
                    cw, logger, fields);
                String name = jdoclass.getFQName();
                ClassReader cr = loadJavaClass(false, name, scp.output, false);
                cr.accept(fam, false);
                writeJavaClass(name, cw, scp.output);
            }
        }
    }

    class DummyAccessorModifier extends LoggedClassAdapter implements Constants {

        private Set methods;

        /**
         * @param fields is a set of field names
         */
        public DummyAccessorModifier(ClassVisitor classVisitor,
                                     Logger logger,
                                     Set fields) {
            super(classVisitor, logger);
            this.methods = new HashSet();
            Iterator i = fields.iterator();
            while (i.hasNext()) {
                SpeedoField f = (SpeedoField)i.next();
                String getter = NamingRules.getterName(f.jdoClass, f.name);
                String setter = NamingRules.setterName(f.jdoClass, f.name);
                methods.add(getter);
                methods.add(setter);
                if (add) {
                    // add dummy getter
                    CodeVisitor _cv = this.cv.visitMethod(ACC_PUBLIC, getter, "()" + f.desc, null, null);
                    _cv.visitInsn(ACONST_NULL);
                    _cv.visitInsn(ARETURN);
                    _cv.visitMaxs(1, 1);
                    // add dummy setter
                    _cv = this.cv.visitMethod(ACC_PUBLIC, setter, "(" + f.desc + ")V", null, null);
                    _cv.visitInsn(RETURN);
                    _cv.visitMaxs(2, 2);
                }
            }
        }

        public CodeVisitor visitMethod(int i, String s, String s1, String[] strings, Attribute attrs) {
            if (!add && methods.contains(s)) {
                // remove dummy getter or setter
                return null;
            } else {
                return super.visitMethod(i, s, s1, strings, attrs);
            }
        }
    }
}
