/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Constants;
import org.objectweb.asm.Attribute;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This class transforms the access of fields
 * @author S.Chassande-Barrioz
 */
public class FieldModifier extends EnhancerComponent {

    public final static String LOGGER_NAME
        = SpeedoProperties.LOGGER_NAME + ".generation.enhancer.fieldMdodifier";

    boolean toProtected;

    public FieldModifier(boolean toProtected) {
        this.toProtected = toProtected;
    }

    public boolean init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
	    return !scp.getXmldescriptor().isEmpty();
    }

    public void process() throws SpeedoException {
        if (scp.getXmldescriptor() == null || scp.getXmldescriptor().isEmpty())
            throw new SpeedoEnhancerException("xmldescriptor not initialised");
        Map fieldModifs = new HashMap();
        Collection xmls = scp.getXmldescriptor().values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itclass = desc.getSpeedoClasses().iterator(); itclass.hasNext();) {
                fieldModifs.clear();
                SpeedoClass jdoclass = (SpeedoClass) itclass.next();
                Iterator it = jdoclass.jdoField.values().iterator();
                while (it.hasNext()) {
                    SpeedoField sf = (SpeedoField) it.next();
                    if (sf.access == 0) {
                        if (toProtected) {
                            fieldModifs.put(sf.name,
                                new FieldModif(sf.name, 0, Constants.ACC_PROTECTED));
                        } else {
                            fieldModifs.put(sf.name,
                                new FieldModif(sf.name, Constants.ACC_PROTECTED, 0));
                        }
                    } else if ((sf.access & Constants.ACC_PRIVATE) != 0) {
                        if (toProtected) {
                            fieldModifs.put(sf.name, new FieldModif(sf.name,
                                    Constants.ACC_PRIVATE, Constants.ACC_PROTECTED));
                        } else {
                            fieldModifs.put(sf.name, new FieldModif(sf.name,
                                    Constants.ACC_PROTECTED, Constants.ACC_PRIVATE));
                        }
                    }
                }
                if (fieldModifs.isEmpty()) {
                    continue;
                }
                ClassWriter cw = new ClassWriter(false);
                FieldAccessModifier fam = new FieldAccessModifier(
                    cw, logger, fieldModifs);
                String name = jdoclass.getFQName();
                ClassReader cr = loadJavaClass(false, name, scp.output, false);
                cr.accept(fam, false);
                writeJavaClass(name, cw, scp.output);
            }
        }
    }

    class FieldAccessModifier extends LoggedClassAdapter {

        private Map fieldModifs;

        /**
         * @param fieldModifs is a map:
         * key=field name
         * value=FieldModif instance
         */
        public FieldAccessModifier(ClassVisitor classVisitor,
                                   Logger logger,
                                   Map fieldModifs) {
            super(classVisitor, logger);
            this.fieldModifs = fieldModifs;
        }

        /**
         * @param access
         * @param name
         * @param desc
         * @param value
         */
        public void visitField(final int access,
                               final String name,
                               final String desc,
                               final Object value,
                               final Attribute attrs) {
            FieldModif fm = (FieldModif) fieldModifs.get(name);
            int ac;
            if (fm != null && (((access & fm.oldAccess) != 0)
                    || access == fm.oldAccess)) {
                logger.log(BasicLevel.DEBUG, "Field " + name
                        + " old:" + access + " / new: " + fm.newAccess);
                ac = access - fm.oldAccess + fm.newAccess;
            } else {
                ac = access;
            }
            super.visitField(ac, name, desc, value, attrs);
        }
    }

    class FieldModif {
        public String name;
        public int oldAccess;
        public int newAccess;

        public FieldModif(String name, int oldAccess, int newAccess) {
            this.name = name;
            this.oldAccess = oldAccess;
            this.newAccess = newAccess;
        }

        public boolean equals(Object obj) {
            return (obj instanceof FieldModif)
                    && name.equals(((FieldModif) obj).name)
                    && oldAccess  == ((FieldModif) obj).oldAccess
                    && newAccess  == ((FieldModif) obj).newAccess;
        }

        public int hashCode() {
            return oldAccess;
        }
    }
}

