/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.asm.Attribute;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.List;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class RedundencyRemover extends LoggedClassAdapter {

    boolean hasDotClassMethod;
    List methodNames ;
    String className;

    public RedundencyRemover(ClassVisitor classVisitor, Logger logger) {
        super(classVisitor, logger);
        hasDotClassMethod = false;
        methodNames = new ArrayList();
    }
    public void visit(final int version, final int access,
                      final String name,
                      final String superName,
                      final String[] interfaces,
                      final String sourceFile) {
        this.className = name;
        super.visit(version, access, name, superName, interfaces, sourceFile);
    }

    public CodeVisitor visitMethod(final int access,
                                   final String name,
                                   final String desc,
                                   final String[] exceptions,
                                   final Attribute attrs) {
        String mn = name;
        if ("class$".equals(name)) {
            if (hasDotClassMethod) {
                if (debug) {
                    logger.log(BasicLevel.DEBUG, "remove useless .class static method");
                }
                return null;
            } else {
                if (debug) {
                    logger.log(BasicLevel.DEBUG, "First .class static method");
                }
                hasDotClassMethod = true;
            }
        } else if ("<clinit>".equals(name)) {
            mn = "clinit$" + methodNames.size();
            methodNames.add(mn);
            if (debug) {
                logger.log(BasicLevel.DEBUG, "rename a static section to " + mn);
            }
        }
        return super.visitMethod(access, mn, desc, exceptions, attrs);
    }

    public void visitEnd() {
        CodeVisitor _cv = this.cv.visitMethod(
                Constants.ACC_STATIC, "<clinit>", "()V", null, null);
        for(int i=0; i<methodNames.size(); i++) {
            String mn = (String) methodNames.get(i);
            _cv.visitMethodInsn(Constants.INVOKESTATIC, className, mn, "()V");
        }
        _cv.visitInsn(Constants.RETURN);
        super.visitEnd();
    }
}
