/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.generation.jdo;

import java.util.Iterator;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.sequence.lib.SpeedoSequence;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;

/**
 * @author Y.Bersihand
 */
public class SequenceGenerator extends AbstractGeneratorComponent {

    public final static String LOGGER_NAME
            = SpeedoProperties.LOGGER_NAME + ".generation.jorm";

	public boolean init() throws SpeedoException {
		if (scp.getXmldescriptor().isEmpty()) {
		    return false;
	    }
		logger = scp.loggerFactory.getLogger(LOGGER_NAME);
		return true;
	}

	public void process() throws SpeedoException {
		if (scp.getXmldescriptor().isEmpty()) {
	        return;
        }
		//add the sequence meta information to serialize for each xml descriptor
        for (Iterator itDesc = scp.getXmldescriptor().values().iterator(); itDesc.hasNext();) {
            addSerializeSequence((SpeedoXMLDescriptor) itDesc.next());
        }
	}
	
	private void addSerializeSequence(SpeedoXMLDescriptor desc)
    throws SpeedoException {
		// Calculate the list of the Jorm Meta Object which must be serialized
		for (Iterator itPack = desc.jdoPackage.values().iterator(); itPack.hasNext();) {
			SpeedoPackage sp = (SpeedoPackage) itPack.next();
			for (Iterator itSequence = sp.jdoSequence.values().iterator(); itSequence.hasNext();) {
				SpeedoSequence jdoSequence = (SpeedoSequence) itSequence.next();
				desc.mos.add(jdoSequence);
			}
			for (Iterator itClass = sp.jdoClass.values().iterator(); itClass.hasNext();) {
				SpeedoClass speedoClass = (SpeedoClass) itClass.next();
				//if the identityType is datastore and id extension is sequence
				if (speedoClass.identityType == SpeedoIdentity.CONTAINER_ID
						&& speedoClass.getExtension(SpeedoProperties.VENDOR_NAME,
							SpeedoProperties.ID).value.equals(SpeedoProperties.ID_SEQUENCE)) {
					//create a new sequence
					SpeedoSequence newSequence = new SpeedoSequence();
					String name;
				    //sequence name
					SpeedoExtension se = speedoClass.getExtension(
				            SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME);
			        if (se != null) {
			        	name = se.value;
			        } else {
			            se = speedoClass.getExtension(
			                SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_NAME);
			            if (se == null) {
			            	name = speedoClass.name.toUpperCase();
			            } else {
			            	name = se.value;
			            }
			            name += "_SEQ";
			        }
			        newSequence.name = name;
			        newSequence.datastoreName = name;
			        
				    //sequence increment
					se = speedoClass.getExtension(
				            SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_INC);
				    if (se != null) {
				    	//set the increment
				    	newSequence.increment = new Integer(se.value);
				    } else {
				    	newSequence.increment = new Integer(1);
				    }

				    // sequence start
					se = speedoClass.getExtension(
				            SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_START);
				    if (se != null) {
				    	//set the start
				    	newSequence.start = new Integer(se.value);
				    }

				    //sequence start
					se = speedoClass.getExtension(
				            SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_CACHE);
				    if (se != null) {
				    	//set the cache
				    	newSequence.cache = new Integer(se.value);
				    }

				    // sequence allocator
					se = speedoClass.getExtension(
				            SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_ALLOCATOR);
				    if (se != null) {
				    	//set the factory class
				    	newSequence.factoryClass = se.value;
				    }
				    //set the package name
				    newSequence.packageName = sp.name;
				    //set a default strategy
				    newSequence.strategy = SpeedoSequence.NON_TRANSACTIONAL;
				    //add the sequence to the package
				    sp.addSequence(newSequence);
				    //add it to the list of sequences to serialize
				    desc.mos.add(newSequence);
				}
			}
		}
	}
}
