/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoTuple;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class AbstractMetaInfoVisitor
		extends AbstractGeneratorComponent
		implements MetaInfoVisitor{

    public final static String LOGGER_NAME =
            SpeedoProperties.LOGGER_NAME + ".generation.mivisitor";

    /**
     * The next MetaInfoVisitor instance. If the field is null that means there
     * is no next. Then the current MetaInfoVisitor is the last.
     */
    MetaInfoVisitor mim;

    /**
     * builds a MetaInfoVisitor which is the last of the chain
     */
    public AbstractMetaInfoVisitor() {
        mim = null;
    }

    /**
     * builds a MetaInfoVisitor which is the last of the chain
     */
    public AbstractMetaInfoVisitor(MetaInfoVisitor mim) {
        this.mim = mim;
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

	public boolean init() throws SpeedoException {
		logger = scp.loggerFactory.getLogger(getLoggerName());
		debug = logger != null && logger.isLoggable(BasicLevel.DEBUG);
		return !scp.getXmldescriptor().isEmpty();
	}

	public void process() throws SpeedoException {
		visitCompilerParameter(scp);
	}
	
    // IMPLEMENTATION OF THE METHODS FROM THE MetaInfoVisitor INTERFACE //
    //------------------------------------------------------------------//

    public MetaInfoVisitor getNext() {
        return mim;
    }

    public void setNext(MetaInfoVisitor next) {
        mim = next;
    }

    public void visitCompilerParameter(SpeedoCompilerParameter scp) throws SpeedoException {
        if (logger == null) {
            setLogger(scp.loggerFactory.getLogger(getLoggerName()));
        }
        if (mim != null) {
            mim.visitCompilerParameter(scp);
        }
    }

    public void visitXml(SpeedoXMLDescriptor xml) throws SpeedoException {
        if (mim != null) {
            mim.visitXml(xml);
        }
    }

    public void visitPackage(SpeedoPackage sp) throws SpeedoException {
        if (mim != null) {
            mim.visitPackage(sp);
        }
    }

    public void visitClass(SpeedoClass sc) throws SpeedoException {
        if (mim != null) {
            mim.visitClass(sc);
        }
    }

    public void visitField(SpeedoField sf) throws SpeedoException {
        if (mim != null) {
            mim.visitField(sf);
        }
    }

    public void visitExtension(SpeedoExtension se) throws SpeedoException {
        if (mim != null) {
            mim.visitExtension(se);
        }
    }

    /**
     * Helper method able to find the SpeedoClass hosting a Speedo meta object.
     * if the specified object is not a child (directly or not) of a SpeedoClass
     * then a null value is returned.
     * @param o is a child (directly or not) of SpeedoClass
     * (supported types: SpeedoClass, SpeedoField, SpeedoTuple, SpeedoExtension)
     */
    protected static SpeedoClass getSpeedoClass(Object o) {
        if (o instanceof SpeedoClass) {
            return (SpeedoClass) o;
        } else if (o instanceof SpeedoField) {
            return ((SpeedoField) o).jdoClass;
        } else if (o instanceof SpeedoTuple) {
            return ((SpeedoTuple) o).jdoField.jdoClass;
        } else if (o instanceof SpeedoExtension) {
            return getSpeedoClass(((SpeedoExtension) o).jdoElement);
        } else {//SpeedoPackage, SpeedoXMLDescriptor, ...
            return null;
        }
    }
}
