/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;

/**
 * Assign a default value for the datastore identifier: polymorphid
 * 
 * @author S.Chassande-Barrioz
 */
public class DataStoreIdAdder extends AbstractMetaInfoVisitor {
    public void visitClass(SpeedoClass sc) throws SpeedoException {
        super.visitClass(sc); 
		if ((sc.identityType == SpeedoIdentity.CONTAINER_ID)
			&& (sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.ID) == null)) {
			//if no sql-seq-name, add an id extension with long
			if (sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME) == null) {
				sc.addExtension(new SpeedoExtension(SpeedoProperties.VENDOR_NAME,
						SpeedoProperties.ID, SpeedoProperties.ID_LONG, sc));
			} else {
				//else, add an id extension with sequence
				sc.addExtension(new SpeedoExtension(SpeedoProperties.VENDOR_NAME,
						SpeedoProperties.ID, SpeedoProperties.ID_SEQUENCE, sc));
			}
		}
		//datastore
		if ((sc.identityType == SpeedoIdentity.CONTAINER_ID)) {
			//no id but a sql-seq-name
			if (sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.ID) == null 
					&& sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME) != null){
				//add a datastoreSequence to the speedo class
				sc.datastoreSequence = sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME).value;
			} else if (sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.ID).value.equals(SpeedoProperties.ID_SEQUENCE)){
				SpeedoExtension se = sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME);
				if (se != null) {
					sc.datastoreSequence = se.value;
				} else {
					se = sc.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_NAME);
		            if (se == null) {
		            	sc.datastoreSequence = sc.name.toUpperCase();
		            } else {
		            	sc.datastoreSequence = se.value;
		            }
		            sc.datastoreSequence += "_SEQ";
				}
			}
		}
	}
}
