/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.enhancer.MetaDataAnalyzer;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;

/**
 * Visit the speedo meta information in order to fill the meta information with
 * the part forgotten by the user.
 *  - ReverseFieldAdder
 * @see ReverseFieldAdder
 * @author S.Chassande-Barrioz
 */
public class MetaInfoVisitors extends AbstractGeneratorComponent {

    public final static String LOGGER_NAME
            = SpeedoProperties.LOGGER_NAME + ".generation.compiler";
    private AbstractMetaInfoVisitor[] modifiers;

    public boolean init() throws SpeedoException {
	    if (scp.getXmldescriptor().isEmpty()) {
		    return false;
	    }
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        modifiers = new AbstractMetaInfoVisitor[] {
            new MetaInfoVisitorImpl(),
            new TupleExtensionCopier(),
            new KeyFieldChecker(),
            new ReverseFieldAdder(),
            new InheritanceVisitor(),
            new MetaDataAnalyzer(),
            new PrimaryKeyFieldAdder(),
            new JavaLangShorcutVisitor(),
			new VersionFieldAdder(),
            new DataStoreIdAdder(),
            new FieldConverterVisitor(),
			new ExtensionVerifier()
        };
        for(int i=1; i<modifiers.length; i++) {
			modifiers[i-1].setSpeedoCompilerParameter(scp);
			modifiers[i-1].init();
            modifiers[i-1].setNext(modifiers[i]);
        }
	    return true;
    }

    public void process() throws SpeedoException {
        modifiers[0].visitCompilerParameter(scp);
    }
}
