<?xml version="1.0"?>
<!--
  Speedo: an implementation of JDO compliant personality on top of JORM generic
  I/O sub-system.
  Copyright (C) 2001-2004 France Telecom R&D
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
 
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  Contact: sebastien.chassandebarrioz@rd.francetelecom.com
 
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
   <!-- Import xalan extensions -->
   <xsl:import href="xalan-ext.xsl"/>

   <!-- Defin string variables -->
   <xsl:param name="request.locale">en</xsl:param>

   <xsl:variable name="strings" select="document(concat('strings_', $request.locale, '.xml'))"/>

   <!-- Common head template -->
   <xsl:template name="head">
      <xsl:if test="$head.title">
         <title>
            <xsl:call-template name="str">
               <xsl:with-param name="id">
                  <xsl:value-of select="$head.title"/>
               </xsl:with-param>
            </xsl:call-template>
         </title>
      </xsl:if>

      <xsl:if test="$html.stylesheet">
         <link rel="stylesheet" href="{$html.stylesheet}"
            type="{$html.stylesheet.type}"/>
      </xsl:if>
   </xsl:template>

   <xsl:template name="mbeanview">
      <xsl:param name="objectname"/>
      <xsl:param name="colspan">1</xsl:param>
      <xsl:param name="text">common.mbeanview.return</xsl:param>

      <tr>
         <td class="darkline" align="right" colspan="{$colspan}">
            <xsl:variable name="objectname-encode">
               <xsl:call-template name="uri-encode">
                  <xsl:with-param name="uri" select="$objectname"/>
               </xsl:call-template>
            </xsl:variable>

            <a href="/mbean?objectname={$objectname-encode}">
               <xsl:call-template name="str">
                  <xsl:with-param name="id">
                     <xsl:value-of select="$text"/>
                  </xsl:with-param>
               </xsl:call-template>
            </a>
         </td>
      </tr>
   </xsl:template>

   <!-- Common tabs template -->
   <xsl:template name="tabs">
      <xsl:param name="selection" select="."/>

      <xsl:variable name="welcome.class">
         <xsl:choose>
            <xsl:when test="$selection='welcome'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="connection.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=connection'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="pmf.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=pmf'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="query.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=query'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="cache.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=cache'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="tx.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=tx'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="log.class">
         <xsl:choose>
            <xsl:when test="$selection='speedo:name=log'">fronttab</xsl:when>
            <xsl:otherwise>backtab</xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <center><table cellpadding="0" cellspacing="0" border="0">
         <tr>
            <td class="{$connection.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dconnection" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.connectionview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>

            <td width="2"/>

            <td class="{$pmf.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dpmf" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.pmfview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>

            <td width="2"/>

            <td class="{$query.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dquery" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.queryview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>

            <td width="2"/>

            <td class="{$cache.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dcache" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.cacheview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>

            <td width="2"/>

            <td class="{$tx.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dtx&amp;operations=false" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.txview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>

            <td width="2"/>

            <td class="{$log.class}">
               <a href="/mbean?objectname=speedo%3Aname%3Dlog&amp;template=monolog" class="tabs">
                  <xsl:call-template name="str">
                     <xsl:with-param name="id">common.tabs.logview</xsl:with-param>
                  </xsl:call-template>
               </a>
            </td>
         </tr>
      </table></center>
      <hr/>
   </xsl:template>

   <xsl:template name="aggregation-navigation">
      <xsl:param name="url"/>
      <xsl:param name="total"/>
      <xsl:param name="step"/>
      <xsl:param name="start"/>
      <xsl:param name="str.prefix">common</xsl:param>

      <xsl:if test="$total&gt;$step">
         <xsl:variable name="isfirst">
            <xsl:choose>
               <xsl:when test='$start=0'>true</xsl:when>
               <xsl:when test='$start&gt;0'>false</xsl:when>
            </xsl:choose>
         </xsl:variable>
         <xsl:variable name="islast">
            <xsl:choose>
               <xsl:when test='$total&lt;=($step + $start)'>true</xsl:when>
               <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <tr>
            <td/>
         </tr>
         <tr>
            <td>
               <xsl:choose>
                  <xsl:when test="$isfirst='false'">
                     <a href="{$url}&amp;start=0">
                        <xsl:call-template name="str">
                           <xsl:with-param name="id">
                              <xsl:value-of select="concat($str.prefix, '.navigation.first')"/>
                           </xsl:with-param>
                        </xsl:call-template>
                     </a>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:call-template name="str">
                        <xsl:with-param name="id">
                           <xsl:value-of select="concat($str.prefix, '.navigation.first')"/>
                        </xsl:with-param>
                     </xsl:call-template>
                  </xsl:otherwise>
               </xsl:choose>
				 -
               <xsl:choose>
                  <xsl:when test="$isfirst='false'">
                     <xsl:variable name="previndex" select="($start - $step)"/>
                     <a href="{$url}&amp;start={$previndex}">
                        <xsl:call-template name="str">
                           <xsl:with-param name="id">
                              <xsl:value-of select="concat($str.prefix, '.navigation.previous')"/>
                           </xsl:with-param>
                        </xsl:call-template>
                     </a>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:call-template name="str">
                        <xsl:with-param name="id">
                           <xsl:value-of select="concat($str.prefix, '.navigation.previous')"/>
                        </xsl:with-param>
                     </xsl:call-template>
                  </xsl:otherwise>
               </xsl:choose>
				 -
               <xsl:choose>
                  <xsl:when test="$islast='false'">
                     <xsl:variable name="nextindex" select="($start + $step)"/>
                     <a href="{$url}&amp;start={$nextindex}">
                        <xsl:call-template name="str">
                           <xsl:with-param name="id">
                              <xsl:value-of select="concat($str.prefix, '.navigation.next')"/>
                           </xsl:with-param>
                        </xsl:call-template>
                     </a>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:call-template name="str">
                        <xsl:with-param name="id">
                           <xsl:value-of select="concat($str.prefix, '.navigation.next')"/>
                        </xsl:with-param>
                     </xsl:call-template>
                  </xsl:otherwise>
               </xsl:choose>
				 -
               <xsl:choose>
                  <xsl:when test="$islast='false'">
                     <xsl:variable name="lastindex" select="($total - ($total mod $step))"/>
                     <a href="{$url}&amp;start={$lastindex}">
                        <xsl:call-template name="str">
                           <xsl:with-param name="id">
                              <xsl:value-of select="concat($str.prefix, '.navigation.last')"/>
                           </xsl:with-param>
                        </xsl:call-template>
                     </a>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:call-template name="str">
                        <xsl:with-param name="id">
                           <xsl:value-of select="concat($str.prefix, '.navigation.last')"/>
                        </xsl:with-param>
                     </xsl:call-template>
                  </xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
      </xsl:if>
   </xsl:template>

   <!-- Finds a string in the strings file by id. It can replace two params -->
   <xsl:template name="str">
      <xsl:param name="id"/>
      <xsl:param name="p0"/>
      <xsl:param name="p1"/>

      <xsl:variable name="str" select="$strings//str[@id=$id]"/>

      <!-- This is a bit lame, should be improved -->
      <xsl:variable name="temp">
         <xsl:call-template name="replace-param">
            <xsl:with-param name="text" select="$str"/>
            <xsl:with-param name="paramText">{0}</xsl:with-param>
            <xsl:with-param name="paramValue">
               <xsl:copy-of select="$p0"/>
            </xsl:with-param>
         </xsl:call-template>
      </xsl:variable>
      <xsl:choose>
         <xsl:when test="$p1">
            <xsl:call-template name="replace-param">
               <xsl:with-param name="text" select="$temp"/>
               <xsl:with-param name="paramText">{1}</xsl:with-param>
               <xsl:with-param name="paramValue">
                  <xsl:copy-of select="$p1"/>
               </xsl:with-param>
            </xsl:call-template>
         </xsl:when>
         <xsl:otherwise>
            <xsl:copy-of select="$temp"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!-- Replaces paramText by paramValue in text -->
   <xsl:template name="replace-param">
      <xsl:param name="text"/>
      <xsl:param name="paramText"/>
      <xsl:param name="paramValue"/>

      <xsl:choose>
         <xsl:when test="contains($text, $paramText)">
            <xsl:copy-of select="substring-before($text, $paramText)"/>
            <xsl:copy-of select="$paramValue"/>
            <xsl:copy-of select="substring-after($text, $paramText)"/>
         </xsl:when>

         <xsl:otherwise>
            <xsl:value-of select="$text"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>
   
	<!-- This function prints a comma list. Each element management are 
	     delegated to the function 'manageElem' with the param 'elem'.
	     param: list is a string comma list (can be empty)-->
	<xsl:template name="tokenizeCommaList">
		<xsl:param name="list"/>

		<xsl:if test="string-length($list) &gt; 0">
			<!-- the list has one element at least -->
		
			<!-- get the first element -->
			<xsl:variable name="elem">
   				<xsl:value-of select="substring-before($list, ',')"/>
			</xsl:variable>
			
			<xsl:if test="string-length($elem) = 0">
				<!-- the list contains only one element -->
				<xsl:call-template name="manageElem">
   					<xsl:with-param name="elem" select="$list"/>
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="string-length($elem) &gt; 0">
				<!-- the list contains several elements -->

				<!-- print the first element -->
				<xsl:call-template name="manageElem">
    				<xsl:with-param name="elem" select="$elem"/>
				</xsl:call-template>

				<!-- recall this function with the end of the list -->
				<xsl:call-template name="tokenizeCommaList">
    				<xsl:with-param name="list" select="substring-after($list, ',')"/>
				</xsl:call-template>
			</xsl:if>
		</xsl:if>
   </xsl:template>
   
</xsl:stylesheet>


