<?xml version="1.0"?>
<!--
  Speedo: an implementation of JDO compliant personality on top of JORM generic
  I/O sub-system.
  Copyright (C) 2001-2004 France Telecom R&D
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
 
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  Contact: sebastien.chassandebarrioz@rd.francetelecom.com
 
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
   <xsl:output method="html" indent="yes" encoding="UTF-8"/>

   <xsl:param name="html.stylesheet">stylesheet.css</xsl:param>
   <xsl:param name="html.stylesheet.type">text/css</xsl:param>
   <xsl:param name="head.title">mbean.title</xsl:param>
   <xsl:include href="common.xsl"/>
   <xsl:include href="mbean_attributes.xsl"/>

   <xsl:param name="request.objectname"/>

   <xsl:template name="parameters" match="Parameter">
      <xsl:param name="class"/>
      <table cellpadding="5" cellspacing="0" border="0">
      <xsl:for-each select="Parameter">
         <xsl:sort data-type="text" order="ascending" select="@id"/>
         <xsl:variable name="type.id" select="concat('type', position()-1)"/>
         <xsl:variable name="name.id" select="concat('value', position()-1)"/>
         <xsl:variable name="type" select="@type"/>
         <tr class="{$class}">
            <td align="left" class="mbean_row">
               <xsl:value-of select="@type"/>
            </td>
            <td align="right" class="mbean_row">
               <xsl:choose>
                  <xsl:when test="@type='java.lang.String'
                     or @type='java.lang.String'
                     or @type='java.lang.Double'
                     or @type='java.lang.Short'
                     or @type='java.lang.Integer'
                     or @type='java.lang.Long'
                     or @type='java.lang.Float'
                     or @type='java.lang.Byte'
                     or @type='java.lang.Boolean'
                     or @type='java.lang.Number'
                     or @type='java.lang.Character'
                     or @type='javax.management.ObjectName'
                     or @type='int'
                     or @type='short'
                     or @type='boolean'
                     or @type='byte'
                     or @type='double'
                     or @type='long'
                     or @type='char'
                     or @type='float'">
                     <xsl:attribute name="valid">true</xsl:attribute>
                     <xsl:call-template name="raw-input">
                        <xsl:with-param name="name" select="$name.id"/>
                        <xsl:with-param name="type" select="$type"/>
                        <xsl:with-param name="value"/>
                        <xsl:with-param name="strinit">false</xsl:with-param>
                     </xsl:call-template>
                  </xsl:when>
                  <xsl:when test="@strinit='true'">
                     <xsl:attribute name="valid">true</xsl:attribute>
                     <xsl:call-template name="raw-input">
                        <xsl:with-param name="name" select="$name.id"/>
                        <xsl:with-param name="type" select="$type"/>
                        <xsl:with-param name="value"/>
                        <xsl:with-param name="strinit">true</xsl:with-param>
                     </xsl:call-template>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:attribute name="valid">false</xsl:attribute>
                     <xsl:call-template name="str">
                        <xsl:with-param name="id">mbean.parameters.unknown</xsl:with-param>
                     </xsl:call-template>
                  </xsl:otherwise>
               </xsl:choose>
               <input type="hidden" name="{$type.id}" value="{$type}"/>
            </td>
         </tr>
      </xsl:for-each>
      </table>
   </xsl:template>

   <xsl:template name="operations">
      <table cellpadding="10" cellspacing="0" border="1">
         <xsl:variable name="invoke.str">
            <xsl:call-template name="str">
               <xsl:with-param name="id">mbean.operations.invoke</xsl:with-param>
            </xsl:call-template>
         </xsl:variable>
         <xsl:for-each select="Operation">
            <xsl:variable name="name">
               <xsl:value-of select="@name"/>
            </xsl:variable>
            <xsl:variable name="classtype">
               <xsl:if test="(position() mod 2)=1">clearline</xsl:if>
               <xsl:if test="(position() mod 2)=0">clearline</xsl:if>
            </xsl:variable>
            <xsl:variable name="hasParameters">
               <xsl:if test="count(./Parameter)>0">true</xsl:if>
               <xsl:if test="count(./Parameter)=0">false</xsl:if>
            </xsl:variable>


            <tr class="{$classtype}">
               <form action="invoke">
                  <input name="operation" type="hidden" value="{$name}"/>
                  <input type="hidden" name="objectname" value="{$request.objectname}"/>
                  <td align="left" class="mbean_row">
                     <xsl:value-of select="@return"/>
					 <xsl:text> </xsl:text>
                     <xsl:value-of select="@name"/>
                  </td>
                  <xsl:if test="$hasParameters='false'">
                     <td align="center" class="mbean_row">
						<xsl:text> </xsl:text>
                     </td>
                     <td align="center" class="mbean_row">
                        <input type="submit" value="{$invoke.str}"/>
                     </td>
                  </xsl:if>
                  <xsl:if test="$hasParameters='true'">
                        <td>
                           <xsl:call-template name="parameters">
                              <xsl:with-param name="class" select="$classtype"/>
                           </xsl:call-template>
                        </td>
                        <td align="center" valign="bottom">
                           <input style="pad-right: 1em;" type="submit" value="{$invoke.str}"/>
                        </td>
                  </xsl:if>
               </form>
            </tr>
         </xsl:for-each>
      </table>
   </xsl:template>

   <!-- Main processing template -->
   <xsl:template match="MBean">
      <html>
         <xsl:call-template name="head"/>
         <body>
			<h1 align="center">Speedo Console</h1>
            <xsl:call-template name="tabs">
               	<xsl:with-param name="selection" >
					<xsl:value-of select="@objectname"/>
				</xsl:with-param>
            </xsl:call-template>
            <xsl:if test="count(./Attribute)>0">
				<p><b>Attributes:</b></p>
    	        <xsl:call-template name="attribute"/>
			</xsl:if>
            <xsl:if test="count(./Operation)>0">
				<p><b>Methods:</b></p>
    	        <xsl:call-template name="operations"/>
			</xsl:if>
         </body>
      </html>
   </xsl:template>
</xsl:stylesheet>

