/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.lib;

import org.objectweb.perseus.persistence.api.StorageManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.NoDSIPersistenceException;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import org.objectweb.perseus.persistence.api.State;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.mim.api.LifeCycle;
import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.mim.api.SpeedoProxy;

import javax.jdo.listener.InstanceLifecycleEvent;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;


/**
 * This class is an implementation of the StorageManager interface based on
 * Jorm.The single hypothesis concerns the architecture of the CacheEntry:
 * - the CacheEntry implements the PBinding interface
 * - the obj parameters implement State and PAceessor.
 *
 * @author S.Chassande-Barrioz
 */
public class JormStorageManager
        implements StorageManager, BindingController {

    public final static String JORM_FACTORY_BINDING = "jorm-factory";
    public final static String LOGGER_NAME = "org.objectweb.jorm.storageManager";

    protected JormFactory jormFactory = null;

    protected Logger logger = null;

	/**
	 * listes for each working set the persistent object removed in other
	 * working set. When a working set want to read a persistent object from
	 * the data support, a prefetch buffer can be used. But the prefetch buffer
	 * can contains a removed object.
	 */
	protected Map ws2removedpo = new HashMap();
	
    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public String[] listFc() {
        return new String[] { JORM_FACTORY_BINDING };
    }

    public Object lookupFc(String s) {
		if (JORM_FACTORY_BINDING.equals(s)) {
            return jormFactory;
        }
        return null;
    }

    public void bindFc(String s, Object o) {
        if ("logger".equals(s)) {
			logger = (Logger) o;
		} else if (JORM_FACTORY_BINDING.equals(s)) {
            jormFactory = (JormFactory) o;
        }
    }

    public void unbindFc(String s) {
		if (JORM_FACTORY_BINDING.equals(s)) {
            jormFactory = null;
        }
    }


	//IMPLEMENTATION OF THE StorageManager INTERFACE //
	//-----------------------------------------------//

    public Object export(ConnectionHolder context, Object obj) throws PersistenceException {
        PBinding pb = (PBinding) obj;
        try {
            if (pb.getStatus() == PBinding.LIFECYCLE_DELTOWRITE) {
                pb.write(context, (PAccessor) obj);
            }
            return pb.export(context);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public Object export(ConnectionHolder context, Object obj, Object hints) throws PersistenceException {
		PBinding pb = (PBinding) obj;
        try {
            if (pb.getStatus() == PBinding.LIFECYCLE_DELTOWRITE) {
                pb.write(context, (PAccessor) obj);
            }
            return pb.export(context, hints);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void unexport(ConnectionHolder context, Object oid) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG,
                    "unexport ctx=" + context + " / oid=" + oid);
        }
        try {
            ((PName) oid).unexport(context);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
	    registerUnexport(context.getWorkingSet(), oid);
    }

    public void unexport(ConnectionHolder context, Object oid, Object hints) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG,
                    "unexport(hints) ctx=" + context + " / oid=" + oid);
        }
        try {
            ((PName) oid).unexport(context, hints);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
	    registerUnexport(context.getWorkingSet(), oid);
    }

    public void read(ConnectionHolder context, Object oid, State obj) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "read ctx=" + context
                    + " / oid=" + oid
                    + " / obj.class=" + obj.getClass().getName());
		SpeedoProxy pb = (SpeedoProxy) obj.getCacheEntry();
		int jdoStatus = ((SpeedoAccessor) obj).jdoGetStatus();
		if (pb.getStatus() == PBinding.LIFECYCLE_DELTOWRITE
		        && jdoStatus != LifeCycle.PERSISTENT_DELETED
		        && jdoStatus != LifeCycle.PERSISTENT_NEW_DELETED) {
		    throw new PersistenceException(
		            "Concurrency problem, transaction must be rolledback");
		}
        try {
            pb.read(context, (PAccessor) obj);
            //modified with rebind
            ((SpeedoAccessor) obj).indexFieldModified(Integer.MAX_VALUE, true);
	        pb.getSpeedoHome().sendEvent(InstanceLifecycleEvent.LOAD, pb, null, false); 
        } catch (PExceptionNoDSI e) {
			if (logger.isLoggable(BasicLevel.DEBUG)) {
	            logger.log(BasicLevel.DEBUG, "read ==> NO DSI");
			}
			throw new NoDSIPersistenceException(e);
		} catch (PException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "read ctx=" + context
	                    + " / oid=" + oid
	                    + " / obj.class=" + obj.getClass().getName(), ie);
            throw new PersistenceException(ie);
        }
    }

    public void read(WorkingSet ws, Object oid, State obj) throws PersistenceException {
        Object conn = ws.getConnectionHolder().getCHConnectionForRead();
		if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "read conn=" + conn
                    + " / oid=" + oid
                    + " / obj.class=" + obj.getClass().getName()
                    + " / tx=" + ws);
		SpeedoProxy pb = (SpeedoProxy) obj.getCacheEntry();
		int jdoStatus = ((SpeedoAccessor) obj).jdoGetStatus();
		if (pb.getStatus() == PBinding.LIFECYCLE_DELTOWRITE
		        && jdoStatus != LifeCycle.PERSISTENT_DELETED
		        && jdoStatus != LifeCycle.PERSISTENT_NEW_DELETED) {
		    throw new PersistenceException(
		            "Concurrency problem, transaction must be rolledback");
		}
		Object ctx = usePrefetchBuffer(ws, oid) ? ws : null;
        try {
            pb.read(conn, (PAccessor) obj, ctx);
            //modified with rebind
            ((SpeedoAccessor) obj).indexFieldModified(Integer.MAX_VALUE, true);
	        pb.getSpeedoHome().sendEvent(InstanceLifecycleEvent.LOAD, pb, null, false); 
		} catch (PExceptionNoDSI e) {
			if (logger.isLoggable(BasicLevel.DEBUG)) {
	            logger.log(BasicLevel.DEBUG, "read ==> NO DSI");
			}
			throw new NoDSIPersistenceException(e);
        } catch (PException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "read conn=" + conn
                    + " / oid=" + oid
                    + " / obj.class=" + obj.getClass().getName()
                    + " / tx=" + ws, ie);
            throw new PersistenceException(ie);
        }
    }

    public void write(ConnectionHolder context, Object oid, State obj) throws PersistenceException {
		SpeedoProxy pb = (SpeedoProxy) obj.getCacheEntry();
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG, "write ctx=" + context
                    + " / oid=" + oid
                    + " / obj.class=" + obj.getClass().getName()
                    + " / binding.status=" + pb.getStatus());
        }
        int s = pb.getStatus();
        if (s == PBinding.LIFECYCLE_ACTIVEFORIO 
                || s == PBinding.LIFECYCLE_NEWTOWRITE
                || s == PBinding.LIFECYCLE_DELTOWRITE) {
            if (s != PBinding.LIFECYCLE_DELTOWRITE) {
    	        pb.getSpeedoHome().sendEvent(InstanceLifecycleEvent.STORE, pb, null, true); 
            }
			try {
				pb.write(context, (PAccessor) obj);
			} catch (PExceptionNoDSI e) {
				throw new NoDSIPersistenceException(e);
			} catch (PException e) {
				throw new PersistenceException(e);
			}
            if (s != PBinding.LIFECYCLE_DELTOWRITE) {
    	        pb.getSpeedoHome().sendEvent(InstanceLifecycleEvent.STORE, pb, null, false); 
            }
		}
    }

	public void beginWS(WorkingSet ws) {
		synchronized(ws2removedpo) {
			ws2removedpo.put(ws, new RemovedPOMemento(ws));
            if (logger.isLoggable(BasicLevel.DEBUG)) {
                logger.log(BasicLevel.DEBUG, "Begin of WS" 
                        + "\n\t-ws=" + ws);
            }
		}
	}

	/**
	 * forget the working set
	 */
	public void endWS(WorkingSet ws) {
	    synchronized(ws2removedpo) {
		    RemovedPOMemento mem = (RemovedPOMemento) ws2removedpo.remove(ws);
		    StringBuffer sb = null;
		    if (mem.removedPOFromWS != null) {
			    for (Iterator it = ws2removedpo.values().iterator(); it.hasNext();) {
			        RemovedPOMemento _mem = (RemovedPOMemento) it.next();
	                _mem.addRemovedPOFromOtherWS(mem.removedPOFromWS);
	                if (logger.isLoggable(BasicLevel.DEBUG)) {
	                    if (sb == null) {
	                        sb = new StringBuffer();
	                        sb.append("Add\n\t-oids= ");
	                        sb.append(mem.removedPOFromWS);
	                    }
	                    sb.append("\n\t-ws= " + _mem.ws);
	                }
			    }                
                if (logger.isLoggable(BasicLevel.DEBUG)) {
                    logger.log(BasicLevel.DEBUG,  sb.toString());
                }
            }
		}
	}

    private void registerUnexport(WorkingSet ws, Object oid) {
		synchronized(ws2removedpo) {
		    ((RemovedPOMemento) ws2removedpo.get(ws)).registerRemovedPOFromWS(oid);
            if (logger.isLoggable(BasicLevel.DEBUG)) {
                logger.log(BasicLevel.DEBUG, "Register the removed object:" 
                        + "\n\t-oid=" + oid
                        + "\n\t-ws=" + ws);
            }
		}
    }
    
	/**
	 * check if the persistent object has not been removed in another context. 
	 * Indeed in case of Jorm uses prefetch buffers, the persistent will be
	 * loaded even if it has been removed from the data base.
	 **/ 
    private boolean usePrefetchBuffer(WorkingSet ws, Object oid) {
		synchronized(ws2removedpo) {
		    boolean res = !((RemovedPOMemento) ws2removedpo.get(ws)).isRemoved(oid);
            if (logger.isLoggable(BasicLevel.DEBUG)) {
                logger.log(BasicLevel.DEBUG,(res ? "": "NOT " ) + "USE prefetch buffer"  
                        + "\n\t-oid=" + oid
                        + "\n\t-ws= " + ws);
            }
		    return res;
		}        
    }
}
/**
 * This structure registers the removed objects from a particular working set.  
 * So an instance of this class has to be associated to a working set.
 *
 * @author S.Chassande-Barrioz
 */
class RemovedPOMemento {
    /**
     * contains the identifiers of persitent objects which have been removed 
     * from the working set linked to this.
     * this field is used at the end of a working set to transmit the list of 
     * removed object from this working set.
     */
    public HashSet removedPOFromWS = null;
    
    /**
     * contains the identifiers of persistent objects which have been removed
     * from the working set linked to this or from another working set.
     * this field is used to choose if it is possible to use a prefetch buffer
     * for a given identifier
     */
    public HashSet removedPO = null;
    
    public final WorkingSet ws;
    
    public RemovedPOMemento(WorkingSet _ws) {
        this.ws = _ws;
    }
    
    /**
     * register the unexport of a persistent object from the linked working set 
     * @param oid is the identifier of the unexported persistent object
     */
    public void registerRemovedPOFromWS(Object oid) {
        if (removedPOFromWS == null) {
            removedPOFromWS = new HashSet();
        }
        removedPOFromWS.add(oid);
        if (removedPO == null) {
            removedPO = new HashSet();
        }
        removedPO.add(oid);
    }
    
    /**
     * Keeps in memory that persistent objects has been removed from another 
     * working set 
     * @param oids is the set of identifier of persistent object which have
     * been unexported in another working set.
     */
    public void addRemovedPOFromOtherWS(Set oids) {
        if (removedPO == null) {
            removedPO = new HashSet();
        }
        removedPO.addAll(oids);
    }
    
    /**
     * Indicates if a persistent object has been removed from the current 
     * working set or another.
     * @param oid is the identifier of the persistent object  which could be 
     * previously removed.
     */
    public boolean isRemoved(Object oid) {
        return removedPO != null && removedPO.contains(oid);
    }
}
