/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.metadata;

import java.util.ArrayList;
import java.util.List;


/**
 * 
 * 
 * @author S.Chassande-Barrioz
 */
public class SpeedoJoin extends SpeedoElement {
	/**
	 * Is the name of table 
	 */
	public String table;
	
	/**
	 * The list of SpeedoColumn composing the join
	 */
	public List columns = new ArrayList();
	
	private final static byte ALL 			= (byte) 0xFF;
	private final static byte INDEXED 		= 1;
	private final static byte OUTER_JOIN 	= 2;
	private final static byte UNIQUE 		= 4;
	private byte conditions = 0;
	
	public final static byte ACTION_NONE 		= 1;
	public final static byte ACTION_NULL 		= 2;
	public final static byte ACTION_CASCADE 	= 3;
	public final static byte ACTION_RESTRICT 	= 4;
	public final static byte ACTION_DEFAULT 	= 5;
	
	public byte deleteAction = ACTION_NONE;

	public void setOuter(boolean v) {
		setCondition(v, OUTER_JOIN);
	}
	public boolean getOuter() {
		return getCondition(OUTER_JOIN);
	}
	
	public void setIndexed(boolean v) {
		setCondition(v, INDEXED);
	}
	public boolean getIndexed() {
		return getCondition(INDEXED);
	}
	
	public void setUnique(boolean v) {
		setCondition(v, UNIQUE);
	}
	public boolean getUnique() {
		return getCondition(UNIQUE);
	}
	
	private void setCondition(boolean v, byte condition) {
		if (v) {
			conditions = (byte) (conditions | condition);
		} else {
			conditions = (byte) (conditions & (ALL - condition));
		}
	}
	private boolean getCondition(byte condition) {
		return (conditions & condition) == 1;
	}
}
