/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.metadata;

import java.util.Iterator;
import java.util.Map;

/**
 * @author S.Chassande-Barrioz
 */
public class SpeedoMetaInfo {

	/**
	 * This fields is used by the Speedo to reference the SpeedoXMLDescriptor.
	 * This field is filled by the JDOParser.
	 * key = jdo File Name
	 * value = SpeedoXMLDescriptor
	 */
	public Map xmldescriptor = null;


	public SpeedoClass getSpeedoClass(String className) {
		return getSpeedoClass(className, (SpeedoXMLDescriptor) null);
	}

	public SpeedoClass getSpeedoClass(String className, SpeedoXMLDescriptor xml) {
		//System.out.println("SMI.getSpeedoClass(" + className + ", " + xml.xmlFile + "): ");
		SpeedoClass sc = null;
		if (xml != null) {
			sc = xml.getSpeedoClass(className, false);
			if (sc != null) {
				//System.out.println("SMI.getSpeedoClass(XML): return " + sc);
				return sc;
			}
		}
		//Try in other xml
		for(Iterator it = xmldescriptor.values().iterator();
		    it.hasNext() && sc == null;) {
			SpeedoXMLDescriptor _xml = (SpeedoXMLDescriptor) it.next();
			if (xml == _xml) {
				continue;
			}
			sc = _xml.getSpeedoClass(className, false);
		}
		//System.out.println("SMI.getSpeedoClass(): return " + sc);
		return sc;
	}

	public SpeedoClass getSpeedoClass(String className, SpeedoPackage sp) {
		//System.out.println("SMI.getSpeedoClass(" + className + ", " + sp.name + "): ");
		SpeedoXMLDescriptor xml = null;
		if (sp != null) {
			xml = sp.jdoXMLDescriptor;
		}
		SpeedoClass sc = getSpeedoClass(className, xml);
		if (sc == null) {
			sc = getSpeedoClass(sp.name + '.' + className, xml);
		}
		//System.out.println("SMI.getSpeedoClass(P): return " + sc);
		return sc;
	}

}
