/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.mim.api;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.perseus.cache.replacement.api.ReplaceableCacheEntry;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import javax.jdo.spi.PersistenceCapable;

/**
 * This interface what is a Proxy in Speedo. A proxy a PersistentCapable
 * instance because it is the user object. It is the jorm PBinding because it is
 * unique. There are several fields: <UL>
 * <LI>The field 'isActive' indicates if the proxy is active (linked to a ProxyManager)</LI>
 * <LI>The fields 'pmf' represents the data source where
 * the current object is persistent.</LI>
 * <LI>The fields 'tpm' is the perseus TransactionalPersistenceManager managing
 * this instance.</LI>
 * </UL>
 *
 * @author	S.Chassande-Barrioz
 */
public interface SpeedoProxy
        extends PersistenceCapable, PBinding, ReplaceableCacheEntry {

    /**
     * Tests whether the Proxy infrastructure has already been set.
     * A proxy has a correct PClassMapping linked to a PBinder,
     * a PName obtained thanks to this PClassMapping.
     *
     * @return true if this object is a Proxy.
     */
    boolean jdoIsActive();

    /**
     * Sets the new value of the proxy flags
     *
     * @see #jdoIsActive()
     * @param newvalue the new value of the proxy flag
     */
    void jdoIsActive(boolean newvalue);
	
    SpeedoHome getSpeedoHome();

    /**
     * It retrieves the SpeedoAccessor instance used in the current context.
     * If the proxy is not active then the reference state is returned.
     * otherwise a $classNameFields is returned
     */
    SpeedoAccessor getSpeedoAccessor();

    /**
     * @return the reference accessor of the SpeedoProxy instance. It can be *
     * null.
     * The Reference accessor is used for non persistent instance, as cache 
     * value and for pessimistic policy. 
     */
    SpeedoAccessor getReferenceAccessor();

    /**
     * Assignes the reference accessor to the SpeedoProxy
     * The Reference accessor is used for non persistent instance, as cache 
     * value and for pessimistic policy.
     *  
     * @param refAcc the new reference accessor
     */
    void setReferenceAccessor(SpeedoAccessor refAcc);

	/**
	 * @return a new SpeedoAccessor instance.
	 */
    SpeedoAccessor createAccessor();

	/**
	 * Copies persistent fields from an accessor to another.
	 * @param src is the accessor containing peristent fields to copy
	 * @param dest is the accessor where persistent fields have to be copied 
	 */
    void copyAccessor(SpeedoAccessor src, SpeedoAccessor dest);

	/**
	 * Set the encoded PName of the detach copy to allow serialization.
	 * @param encodedPName : the encoded PName of the persistent object
	 */
	void setEncodedPName(Object encodedPName);
	
	/**
	 * @return the encoded PName of the detached copy
	 */
	Object getEncodedPName();
	
    /**
     * @return The file name of the jdo descriptor. This file must be availlable
     * in the class at runtime.
     */
    String jdoGetJdoFileName();

    /**
     * Retrieves the hints needed to build the PName of the persistent object
     * during the export action.
     */
    Object jdoGetPNameHints();

    /**
     * Adds an element in the gen class field whose number is given. This method
     * must call the speedoAdd method of the SpeedoGenClassCoherence interface,
     * or the Collection.add method (if the field contains a collection that
     * does implement the previous interface). If the field is null, a new
     * (speedo) collection must be created first.
     */
    void speedoAdd (Object elem, int gcFieldNumber);
}
