/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.mim.lib;

import java.util.Arrays;
import java.util.Collection;
import java.util.Set;
import java.util.HashSet;

import javax.jdo.FetchPlan;


/**
 * @author Y.Bersihand
 */
public class SpeedoFetchPlan implements FetchPlan {

	/**
	 * The list of groups contained in the fetch plan.
	 */
	private Set groups = new HashSet();

	private int fetchSize;
	
	public SpeedoFetchPlan(){
		groups.add(FetchPlan.DEFAULT);	
	}

	public SpeedoFetchPlan(String fgName){
		groups.add(fgName);	
	}
	
	/**
	 * Add a group into the fetch plan.
	 */
    public FetchPlan addGroup(String fetchGroupName){
        if (fetchGroupName != null){
            groups.add(fetchGroupName);
        }
        return this;
    }
	
	/**
	 * Remove a group from the fetch plan.
	 */
    public FetchPlan removeGroup(String fetchGroupName){
    	if (fetchGroupName != null){
    		groups.remove(fetchGroupName);
    	}
        return this;
    }

    public Collection getGroups(){
        return groups;
    }

    public FetchPlan setGroups(Collection fetchGroupNames){
        groups.clear();
        if (fetchGroupNames != null){
            groups.addAll(fetchGroupNames);
        }
        return this;
    }

    public FetchPlan clearGroups() {
        groups.clear();
        groups.add(FetchPlan.DEFAULT);
        return this;
    }
    public FetchPlan setGroup(String fetchGroupName) {
        groups.clear();
        if (fetchGroupName != null){
            groups.add(fetchGroupName);
        }
        return this;
    }
    public FetchPlan setGroups(String[] fetchGroupNames) {
        groups.clear();
        if (fetchGroupNames != null){
            groups.addAll(Arrays.asList(fetchGroupNames));
        }
        return this;
    }

    public FetchPlan setFetchSize(int fetchSize){
        this.fetchSize = fetchSize;
        return this;
    }

	public int getFetchSize(){
        return fetchSize;
    }
}
