/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.speedo.naming.api.NamingManager;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.pm.api.ProxyManagerFactory;
import org.objectweb.jorm.facility.naming.olongid.LongIdManager;
import org.objectweb.jorm.facility.naming.olongid.LongIdBinder;
import org.objectweb.jorm.facility.naming.olongid.LongIdPName;
import org.objectweb.jorm.facility.naming.olongid.LongIdPNC;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.CommonClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.NameRef;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.lib.DivideBy;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Round;

import java.util.Properties;
import java.util.Map;
import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class OLongIdNamingManager
	extends LongIdManager
	implements NamingManager {

	private final static String _LONG_ID_NAME
		= "org.objectweb.jorm.facility.naming.olongid.LongId";
	private final static String _LONG_ID_LID = "olid";
	private final static String BINDER_FOR_CLASS = "c";
	private final static String BINDER_FOR_GENCLASS = "gc";
	private final static String HIDDEN_LID_FIELD_NAME = "jdolid";

	private Logger logger;
	private PMapper mapper;
	private ProxyManagerFactory pmf;

	protected String LONG_ID_NAME() {
	    return _LONG_ID_NAME;
	}
	
	protected String LONG_ID_LID() {
	    return _LONG_ID_LID;
	}
	
	protected PType getFieldType() {
	    return PTypeSpace.OBJLONG;
	}
	
	protected BasicOperand getBasicOperand() {
		return new BasicOperand(new Long(1l << 44), PTypeSpace.OBJLONG);
	}
	
	protected boolean checkFieldType(String type) {
	    return "Long".equals(type) || "java.lang.Long".equals(type);
	}
	
	
	public void init() throws PException {
		if (getMapper() != null) {
			return;
		}
		if (mapper == null) {
			throw new PException("No mapper assigned");
		}
		super.setPMapper(mapper);
		mapper = null;
	}

	// IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
	//---------------------------------------------------------------//
	public void setPMapper(PMapper mapper) throws PException {
		this.mapper = mapper;
	}

	public void setPmf(ProxyManagerFactory pmf) {
		
	}
	
	public boolean supportPNamingcontext() {
		return true;
	}

	public boolean canManage(SpeedoClass sc) {
		if (sc.identityType == SpeedoIdentity.CONTAINER_ID && sc.datastoreSequence == null) {
			SpeedoExtension se = sc.getExtension(
					SpeedoProperties.VENDOR_NAME, SpeedoProperties.ID);
			return se != null
				&& SpeedoProperties.ID_OLONG.equalsIgnoreCase(se.value);
		} else {
			return false;
		}
	}

	public boolean canProvidePBinder(Object hints, ClassLoader classLoader) {
		return NamingManagerHelper
			.getBinderClassNameFromHints(hints, LONG_ID_LID()) != null;
	}

	public boolean canProvidePNamingContext(Object hints, ClassLoader classLoader) {
		return NamingManagerHelper
			.getPNCClassNameFromHints(hints, LONG_ID_LID()) != null;
	}

	public Object encode(PName pn) throws PException {
		if (pn instanceof LongIdPName) {
			return pn.getPNameManager().getPType().getJormName()
					+ SEP + pn.encodeLong();
		}
		return null;
	}

    public PName decode(PNameCoder pnc,
						Object oid,
						Class clazz,
						JormFactory jf) throws PException {
        if (oid instanceof String) {
            String stroid = (String) oid;
            int idx = stroid.indexOf(SEP);
            if (pnc != null) {
                if (pnc instanceof LongIdBinder
                        || pnc instanceof LongIdManager) {
					String idStr = stroid;
                    if (idx != -1) {
                        //The oid contains the class name
						idStr = stroid.substring(idx + SEP.length());
                    }
					return pnc.decodeOlong(Long.valueOf(idStr));

                } else {
                    //The pnc is not a well known pnc, then the oid cannot be managed
                    return null;
                }
            } else {
                //No pnc specified
                if (idx != -1) {
                    //The oid contains the class name
                    String fqcn = stroid.substring(0, idx);
                    ClassLoader cl = getClass().getClassLoader();
                    if (cl == null) {
                        cl = ClassLoader.getSystemClassLoader();
                    }
                    try {
                        pnc = jf.getPNamingContext(fqcn, cl);
                    } catch (Exception e) {
						return null;
                    }
                    if (pnc instanceof LongIdBinder
                            || pnc instanceof LongIdPNC) {
						return pnc.decodeOlong(
							Long.valueOf(
								stroid.substring(idx + SEP.length())));
					} else {
						return null;
					}
                } else {
                    //The oid cannot be managed
                    return null;
                }
            }
        }
        return null;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        return "null";
    }

    public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
		return "new Long(proxy.getPName().encodeLong())";
	}

    public NamingManager.NamingField[] getNamingfields(SpeedoClass sc) {
		Iterator it = sc.jdoField.values().iterator();
		while (it.hasNext()) {
			SpeedoField sf = (SpeedoField) it.next();
			if (sf.primaryKey) {
				return new NamingField[]{
					new NamingField(sf.name, Long.class, "lid", getFieldType())};
			}
		}
		return null;
    }

	public void fillNameDef(MIBuilderHelper mibh,
							Manager manager,
							NameDef nd,
							SpeedoClass tsc,
							SpeedoClass ssc,
							MetaObject mo,
							Reference ref,
							CommonClassMapping hcm,
							JormMIMappingBuilder mb,
							boolean isIdentifier,
							boolean isInGenClass,
							boolean createField,
							Collection createdMOs) throws SpeedoException, PException {
		SpeedoField sf = fetchUniqueLongPKField(tsc);
		boolean debug = logger.isLoggable(BasicLevel.DEBUG);
		if (debug) {
			logger.log(BasicLevel.DEBUG, "LongIdNaming: Filling the name def: "
				+ "\n\tsource class=" + (ssc == null ? null : ssc.getFQName())
				+ "\n\ttarget class=" + (tsc == null ? null : tsc.getFQName())
				+ "\n\tmo=" + mo
				+ "\n\treference=" + ref
				+ "\n\tisIdentifier=" + isIdentifier
				+ "\n\tisInGenClass=" + isInGenClass
				+ "\n\tcreateField=" + createField
				+ "\n\tsf=" + sf);
		}
		PrimitiveElement lid;
		String lidname;
		if (isInGenClass || sf == null || !isIdentifier) {
			String prefix = mibh.getNameDefFieldPrefix(ref, isIdentifier, isInGenClass);
			lidname = prefix + HIDDEN_LID_FIELD_NAME;
		} else {
			createField = false;
			lidname = sf.name;
		}
		logger.log(BasicLevel.DEBUG, "lidname=" + lidname);
		if (createField) {
			lid = mibh.createNameDefField(mo, lidname, getFieldType());
		} else {
			lid = mibh.getPrimitiveField(mo, lidname);
			if (lid == null) {
				throw new SpeedoException(
					mibh.getErrorMessage(tsc, mo, ref)
					+ " Impossible to get the field '" + lidname + "'");
			}
		}
		if ((isInGenClass && isIdentifier)
			|| (!isIdentifier && !isInGenClass && ref instanceof GenClassRef)) {
			//Gen class identifier or gen class reference are based on basid
			nd.setFieldName(lidname);
		} else {
			//class identifier or class reference are based on LongId
			//fetch the composite name
			CompositeName cn = manager.getCompositeName(LONG_ID_NAME());
			if (cn == null) {
				cn = manager.createCompositeName(LONG_ID_NAME());
				ScalarField cnf = cn.createCompositeNameField(
					"lid", getFieldType(), PType.NOSIZE, PType.NOSIZE);
				cnf.setIsAutoCalculated(true);
			}
			//create the nameRef
			NameRef nr = nd.createNameRef(cn);
			nr.addProjection("lid", lid.getName());
			if (isIdentifier) {
				assignDummyKeyToFamilly(ssc.jormclass, nd);
				if (ssc.jormclass.getSuperClasses().isEmpty()) {
					Expression expr = new Round(new DivideBy(
						new BasicParameterOperand(getFieldType(), "lid"),
						getBasicOperand()));
					try {
						expr.compileExpression();
					} catch (ExpressionException e) {
						throw new SpeedoException("Impossible to compile the filter of longid naming", e);
					}
					ssc.jormclass.setInheritanceFilter(nd, expr);
				}
			}
		}
		if (mb != null && createField) {
			mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
		}
	}

	private void assignDummyKeyToFamilly(org.objectweb.jorm.metainfo.api.Class clazz, NameDef nd) {
		clazz.setInheritanceNamingKey(nd, "0");
		for(Iterator it = clazz.getSubClasses().iterator(); it.hasNext();) {
			assignDummyKeyToFamilly(((org.objectweb.jorm.metainfo.api.Class) it.next()), nd);
		}
	}

	private SpeedoField fetchUniqueLongPKField(SpeedoClass tsc) throws SpeedoException {
		Iterator it = tsc.jdoField.values().iterator();
		SpeedoField sf = null;
		ArrayList al = null;
		while (it.hasNext()) {
			SpeedoField _sf = (SpeedoField) it.next();
			if (_sf.primaryKey) {
				if (sf != null) {
					if (al == null) {
						al = new ArrayList();
					}
					al.add(_sf.name);
				} else {
					sf = _sf;
				}
			}
		}
		if (al != null) {
			al.add(sf);
			throw new SpeedoException(
				"Impossible to use an auto incremented identifier if " +
				"several fields have been marked as primary-key " + al
				+ " in the class '" + tsc.getFQName() + "'.");
		}
		if (sf != null) {
		    String t = sf.type();
		    
			if (!checkFieldType(t)) {
				throw new SpeedoException(
					"Impossible to use an auto incremented identifier: " +
					"the field type of '" + sf.name + "' is '" + sf.type()
					+ "' and '" + getFieldType().getJavaName() 
					+ "' is expected (class '"
					+ tsc.getFQName() + "'.");
			}
		}
		return sf;
	}

	public void getJormNamingConfig(NameDef nd,
									SpeedoClass targetClass,
									MetaObject sourceMO,
									String key,
									Properties result) {

		StringBuffer sb = new StringBuffer();
		sb.append(LONG_ID_LID());
		sb.append(NamingManagerHelper.HINTS_SEP);
		boolean isGCR = sourceMO instanceof GenClassRef;
		String binderInfo = (isGCR ? BINDER_FOR_GENCLASS : BINDER_FOR_CLASS);
		sb.append(binderInfo);

		SpeedoClass ancestor = targetClass.getAncestor();
		String ancestorClassName;
		String pncClassName;
		if (!isGCR //For a Genclass ref or genclass identifier
			&& (ancestor != null // has supper class
				|| !targetClass.jormclass.getSubClasses().isEmpty()) // has sub class
			) {
			if (ancestor == null) {
				ancestor = targetClass;
			}
			ancestorClassName = ancestor.getFQName();
			pncClassName = NamingRules.kfpncName(ancestorClassName);
		} else {
			ancestorClassName = targetClass.getFQName();
			pncClassName = binderInfo;
		}
		//specify the pnc className
		sb.append(NamingManagerHelper.HINTS_SEP);
		sb.append(pncClassName);

		//specify the className of the ancestor
		sb.append(NamingManagerHelper.HINTS_SEP);
		sb.append(ancestorClassName);
		result.setProperty(key, sb.toString());
    }

	public PBinder getPBinder(String className,
							  String hints,
							  ClassLoader classLoader,
							  byte mappingStructureRule,
							  Map cn2binder,
							  Map cn2pnc) throws PException {
		init();
		String binderInfo = NamingManagerHelper.getBinderClassNameFromHints(hints, LONG_ID_LID());
		return getPBinder(className, binderInfo);
	}

	public PNamingContext getPNamingContext(String className,
											String hints,
											ClassLoader classLoader,
											byte mappingStructureRule,
											Map cn2binder,
											Map cn2pnc,
											Manager miManager,
											PClassMapping pcm) throws PException {
		init();
		String[] tokens = NamingManagerHelper.getTokens(hints);
		PNamingContext pnc = null;
		if (tokens[NamingManagerHelper.BINDER_IDX].equals(tokens[NamingManagerHelper.PNC_IDX])) {
			//The binder must be used as PNamingContext
			pnc = (PNamingContext) cn2binder.get(className);
			if (pnc == null) {
				//instanciate the binder/PNC
				pnc = (PNamingContext) getPBinder(
					className, tokens[NamingManagerHelper.BINDER_IDX]);
				//register the binder/PNC as binder
				cn2binder.put(className, pnc);
			}
			return pnc;
		} else {
			boolean register = false;
			if (!tokens[NamingManagerHelper.PCLASS_IDX].equals(className)) {
				//The PNC of THE ancestor must be used
				pnc = (PNamingContext) cn2pnc.get(tokens[NamingManagerHelper.PCLASS_IDX]);
				if (pnc == null) {
					register = true;
				}
			}
			if (pnc == null) {
				//instanciate the PNC
				pnc = newClassPNamingContext();
				if (register) {
					cn2pnc.put(tokens[NamingManagerHelper.PCLASS_IDX], pnc);
				}
			}
		}
		return pnc;
	}

	private PBinder getPBinder(String className, String binderInfo) throws PException {
		if (binderInfo.startsWith(BINDER_FOR_GENCLASS)) {
			return newGenClassPBinder();
		} else if (binderInfo.startsWith(BINDER_FOR_CLASS)) {
			return newClassPBinder(className, null);
		} else {
			throw new PException("Bad binder info: " + binderInfo);
		}
	}

	public void setCache(CacheManager cache) {
	}
}
