/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.jdo.datastore.Sequence;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.facility.naming.generator.LongGen;
import org.objectweb.jorm.facility.naming.rdbsequence.RdbSequenceBinder;
import org.objectweb.jorm.facility.naming.rdbsequence.RdbSequencePName;
import org.objectweb.jorm.metainfo.api.CommonClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.naming.api.NamingManager;
import org.objectweb.speedo.sequence.lib.SpeedoSequence;
import org.objectweb.speedo.sequence.lib.SpeedoSequenceBinder;
import org.objectweb.speedo.tools.StringReplace;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceNamingManager
	extends NamingManagerHelper
	implements NamingManager {

    private final static String SEQUENCE_ID ="sid";
    private final static String BINDER_CLASS_NAME
            = RdbSequenceBinder.class.getName();
    private final static String SPEEDO_BINDER_CLASS_NAME 
			= SpeedoSequenceBinder.class.getName(); 
    public final static int SEQ_NAME_IDX 			= 4;
	public final static int SEQ_INCREMENT_IDX 		= 5;
	public final static int SEQ_STARTID_IDX 		= 6;
	public final static int SEQ_CACHE_IDX 			= 7;
	public final static int SEQ_ALLOCATOR_IDX 		= 8;



    public boolean canManage(SpeedoClass sc) {
    	if (sc.datastoreSequence != null) {
    		return true;
    	}
    	return false;
    }

    // IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
    //---------------------------------------------------------------//

    public PName decode(PNameCoder pnc,
                        Object oid,
                        Class clazz,
                        JormFactory jf) throws PException {
        if (oid instanceof String) {
            String stroid = (String) oid;
            if (pnc != null) {
                if (pnc instanceof RdbSequenceBinder) {
					int idx = stroid.indexOf(SEP);
                    if (idx != -1) {
                        //The oid contains the class name
                        return pnc.decodeString(stroid.substring(idx + SEP.length()));
                    } else {
                        //The oid must decoded directly
                        return pnc.decodeString(stroid);
                    }

                } else {
                    //The pnc is not a RdbSequenceBinder, then the oid cannot be managed
                    return null;
                }
            } else {
                //No pnc specified
				int idx = stroid.indexOf(SEP);
	            ClassLoader cl = null;
	            String fqcn;
                if (idx != -1) {
                    //The oid contains the class name
                    fqcn = stroid.substring(0, idx);
                } else if (clazz != null) {
	                fqcn = clazz.getName();
                } else {
	                //The oid cannot be managed
	                return null;
                }
				cl = jf.getClass().getClassLoader();
				if (cl == null) {
					cl = ClassLoader.getSystemClassLoader();
				}
				try {
					pnc = jf.getPBinder(fqcn, cl);
				} catch (Exception e) {
					return null;
				}
				return (pnc instanceof RdbSequenceBinder
					? pnc.decodeString(stroid.substring(idx + SEP.length()))
					: null);
            }
        } else if (pnc instanceof RdbSequenceBinder){
            if (oid instanceof Integer) {
                pnc.decodeLong(((Integer) oid).intValue());
            } else if (oid instanceof Long) {
                pnc.decodeLong(((Long) oid).longValue());
            }
        }
        return null;
    }

    public Object encode(PName pn) throws PException {
        if (pn instanceof RdbSequencePName) {
            return pn.getPNameManager().getPType().getJormName()
                    + SEP + pn.encodeString();
        }
        return null;
    }

	protected String getName() {
		return SEQUENCE_ID;
	}

    public PBinder getPBinder(String className,
							  String hints,
							  ClassLoader cl,
							  byte mappingStructureRule,
                              Map cn2binder,
                              Map cn2pnc) throws PException {
	    SpeedoSequenceBinder binder = (SpeedoSequenceBinder) super.getPBinder(
			className, hints, cl, mappingStructureRule, cn2binder, cn2pnc);
	    String[] tokens = getTokens(hints);
		if (tokens[SEQ_NAME_IDX].length() > 0) {
			if (logger != null && logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "Sequence name: " + tokens[SEQ_NAME_IDX]);
			}
			String sequencePrefix = (className.lastIndexOf('.') != -1) ? className.substring(0, className.lastIndexOf('.')+1):"";
			sequencePrefix = StringReplace.replaceString("/", "", sequencePrefix);
			Sequence sequence = pmf.getSequenceManager().getSequence(sequencePrefix + tokens[SEQ_NAME_IDX]);
			if (sequence == null) {
				throw new PException("Wrong sequence name: " + sequencePrefix + tokens[SEQ_NAME_IDX]);
			}
			LongGen longGen = ((SpeedoSequence) sequence).getLongGen();
			/*if (longGen instanceof RdbSequenceLongGen) {
				((RdbSequenceLongGen) longGen).setLogger(logger);
				binder.setSequenceHelper((RdbSequenceLongGen) longGen);
			}*/
			binder.setSequence(sequence);
		}
        return binder;
    }

	public void getJormNamingConfig(NameDef nd,
									SpeedoClass targetClass,
									MetaObject sourceMO,
									String key,
									Properties result) throws SpeedoException{
	    StringBuffer sb = new StringBuffer();
		sb.append(SEQUENCE_ID);

		sb.append(HINTS_SEP);
		sb.append(SPEEDO_BINDER_CLASS_NAME);

		SpeedoClass ancestor = targetClass.getAncestor();
		String ancestorClassName;
		String pncClassName;
		if (!(sourceMO instanceof GenClassRef) //For a Genclass ref or identifier
			&& (ancestor != null // has supper class
				|| !targetClass.jormclass.getSubClasses().isEmpty()) // has sub class
			) {
			if (ancestor == null) {
				ancestor = targetClass;
			}
			ancestorClassName = ancestor.getFQName();
			try {
				if (targetClass.jormclass.getSubClasses().isEmpty()) {
					pncClassName = SPEEDO_BINDER_CLASS_NAME;
				} else 	if (targetClass.jormclass.detectFilterElementNotInPK(targetClass.jormclass.getInheritanceFilter(nd), nd)) {
					//if has child(ren) 
					//and one of the fields of the filter is not in the primary key,
					//use a PolymorphicPNC
					pncClassName = POLYMORPHIC_PNC;
				} else {
					//use a kfpnc
					pncClassName = NamingRules.kfpncName(ancestor.getFQName());
				}
			} catch (Exception e) {
				logger.log(BasicLevel.ERROR, "Error while retrieving the inheritance filter of the namedef:" + nd.toString());
				throw new SpeedoException("Error while retrieving the inheritance filter of the namedef:" + nd.toString(), e);
			}
		} else {
			ancestorClassName = targetClass.getFQName();
			pncClassName = SPEEDO_BINDER_CLASS_NAME;
		}
		//specify the pnc className
		sb.append(HINTS_SEP);
		sb.append(pncClassName);

		//specify the className of the ancestor
		sb.append(HINTS_SEP);
		sb.append(ancestorClassName);


	    //sequence name
	    sb.append(HINTS_SEP);
	    if (targetClass.datastoreSequence != null) {
	    	sb.append(targetClass.datastoreSequence);
	    }
        result.setProperty(key, sb.toString());
     }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        return "null";
    }

    public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
		return "proxy.getPName()";
    }

    /**
     * The auto incremented field used as identifier can be a visble or
     * an hidden field. In case the field is visible, the
     */
    public void fillNameDef(MIBuilderHelper mibh,
                            Manager manager,
                            NameDef nd,
                            SpeedoClass tsc,
                            SpeedoClass ssc,
                            MetaObject mo,
                            Reference ref,
                            CommonClassMapping hcm,
                            JormMIMappingBuilder mb,
                            boolean isIdentifier,
                            boolean isInGenClass,
                            boolean createField,
                            Collection createdMOs) throws SpeedoException, PException {
        Iterator it = tsc.jdoField.values().iterator();
        SpeedoField sf = null;
        ArrayList al = null;
        while(it.hasNext()) {
            SpeedoField _sf = (SpeedoField) it.next();
            if (_sf.primaryKey) {
                if (sf != null) {
                    if (al == null) {
                        al = new ArrayList();
                    }
                    al.add(_sf.name);
                } else {
                    sf = _sf;
                }
            }
        }
        if (al != null) {
            al.add(sf);
            throw new SpeedoException(
                    "Impossible to use an auto incremented identifier if " +
                    "several fields have been marked as primary-key " + al
                    + " in the class '" + tsc.getFQName() + "'.");
        }
        String fieldName = null;
        fieldName = mibh.getNameDefFieldPrefix(ref, isIdentifier, isInGenClass);
        PType newLongFieldType = PTypeSpace.OBJLONG;
		if (sf == null) {
            fieldName += "id_";
        } else {
			if ("java.lang.Long".equals(sf.type()) || "Long".equals(sf.type())) {
				newLongFieldType = PTypeSpace.OBJLONG;
			} else if ("long".equals(sf.type())) {
				newLongFieldType = PTypeSpace.LONG;
			} else {
                throw new SpeedoException(
                        "Impossible to use an auto incremented identifier: " +
                        "the field type of '" + sf.name + "' is '" + sf.type()
                        + "' and 'java.lang.Long' or long is expected (class '"
                        + tsc.getFQName() + "'.");
            }
            if (!isInGenClass && fieldName.length() == 0) {
                //The field already exists and its mapping is defined when primitive
                // fields are mapped.
                createField = false;
            }
            fieldName += sf.name;
        }
        PrimitiveElement id = null;
        if (createField) {
            id = mibh.createNameDefField(mo, fieldName, newLongFieldType);
        } else {
            id = mibh.getPrimitiveField(mo, fieldName);
            if (id == null) {
                throw new SpeedoException(
                        mibh.getErrorMessage(tsc, mo, ref)
                        + " Impossible to get the field '" + fieldName + "'");
            }
        }
        nd.setFieldName(fieldName);
        if (createField && (mb != null)) {
            mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
        }
    }

    public NamingField[] getNamingfields(SpeedoClass sc) throws PException {
        Iterator it = sc.jdoField.values().iterator();
        while(it.hasNext()) {
            SpeedoField sf = (SpeedoField) it.next();
            if (sf.primaryKey) {
				Class fieldType;
				PType fieldPType;
				if ("java.lang.Long".equals(sf.type()) || "Long".equals(sf.type())) {
					fieldType = Long.class;
					fieldPType = PTypeSpace.OBJLONG;
				} else if ("long".equals(sf.type())) {
					fieldType = Long.TYPE;
					fieldPType = PTypeSpace.LONG;
				} else {
					throw new PException(
							"Impossible to use an auto incremented identifier: " +
							"the field type of '" + sf.name + "' is '" + sf.type()
							+ "' and 'java.lang.Long' or 'long' is expected (class '"
							+ sc.getFQName() + "'.");
				}
                return new NamingField[] {
                    new NamingField(sf.name, fieldType, "lid", fieldPType)};
            }
        }
        return null;
    }
}
