/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 23 mars 2005
 *
 */
package org.objectweb.speedo.usercache.api;

import java.util.Collection;

/**
 * Defines a Manager of a set of user caches. It permits to initalizes user 
 * caches. Each user cache is identifier by an int value. The particular '-1' 
 * value represents no user cache. 
 *
 * @author S.Chassande-Barrioz
 */
public interface UserCacheManager {

	/**
	 * Retrives a user cache
	 * @param cacheId is the identifier of the asked user cache
	 * @return the user cache instance if found, null otherwise
	 */
	UserCache getUserCache(int cacheId);
	
	/**
	 * Retrieves the name of the user cache which index is exactly composed
	 * by some field names.
	 * @param fieldNames is the collection of field names (String). The order of
	 * the collection is not important here.
	 * @return an user cache name if it has been found, null otherwise.
	 */
	UserCache getUserCache(Collection fieldNames);
	
	/**
	 * Retrieves the list of user cache which has been initialized.
	 * @return a collection of UserCache instance
	 */
	Collection getActiveUserCache();
	
	/**
	 * An entry of user cache has been unbound from the real cache.
	 * A typical implementation, undinds from active user caches, the entry.
	 * To do this the a persistent object which has
	 * been previously unexported.
	 * 
	 * @param oid is the identifier of the persistent object to unbind
	 */
	void userCacheEntryUnbound(Object oid);
	
	/**
	 * Initializes the user cache
	 * @param cacheName is the name of the user cache to initialize
	 * @return indicates if the cache has been initialized correctly
	 */
	boolean activeUserCache(String cacheName);
}
