/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.wizard.generation;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.lib.JormCompiler;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.api.SpeedoXMLError;
import org.objectweb.speedo.generation.jorm.JormMIBuilder;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoModifier;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class JormWriter extends AbstractGeneratorComponent {

   public final static String LOGGER_NAME =
            SpeedoProperties.LOGGER_NAME + "wizard.generation";

    /**
     * Jorm classes intended for JORM generation
     */
    protected JormCompiler jormcompiler;


    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public boolean init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        jormcompiler = new JormCompiler();
        JormCompilerParameter jcp = jormcompiler.getCompilerParameter();
        JormCompilerConfigurator jcc = jormcompiler.getCompilerConfigurator();
        try {
            logger.log(BasicLevel.DEBUG, "JormConfigurator: configure() -- get jorm.mapper.list");
            jcc.configure();
            logger.log(BasicLevel.DEBUG, "JormCompilerParameter: loadConfFile()");
            jcp.loadConfFile(jcc.getGlobalJormcOptsFile(), jcc.knownMappers());
            jcc.setLoggerFactory(scp.loggerFactory);
            jcp.setProjectName(scp.projectName);
            jcp.setBindingInheritance("%p%c");
            jcp.setOutput(scp.pdoutput);
            jcp.setBindingAbstract(true);
            jcp.setDtdLocations(scp.dtdLocations);
            jcp.setInputFiles(scp.jorm);
            DirJavaExplorer pe = new DirJavaExplorer();
            pe.setLogger(scp.loggerFactory.getLogger("org.objectweb.jorm.io.pathexplorer"));
            pe.addPath(scp.jormclasspath);
            jcp.setClasspath(pe);
            jcp.setGeneratedWithMapperPackage(false);
            if ((scp.projectName == null) || (scp.mapperName == null))
                return true;
            int idx = scp.mapperName.indexOf('.');
            if (idx == -1) {
                jcc.removeMapper(scp.mapperName);
                jcc.addSubMapper(scp.mapperName, "generic");
            } else {
                String actualmn = scp.mapperName.substring(0,idx);
                jcc.removeMapper(actualmn);
                jcc.addSubMapper(actualmn, scp.mapperName.substring(idx + 1));
            }
            Collection c = jcc.getSubMappers("rdb");
            logger.log(BasicLevel.DEBUG, "jormcOpts file:" + jcc.getJormcOptsFile());
            logger.log(BasicLevel.DEBUG, "rdb sub mappers: " + c);

        } catch (PException e) {
            logger.log(BasicLevel.ERROR,
                    "Impossible to configure Jorm", ExceptionHelper.getNested(e));
            throw new SpeedoException("Impossible to configure Jorm", e);
        }
        return true;
    }

    /**
     * Performs the JORM objects' generation.
     */
    public void process() throws SpeedoException {

        try {
            //Scans the collection of existing .pd files
            Collection c = jormcompiler.getCompilerParameter().getInputFiles();
            if ((c != null) && (logger.isLoggable(BasicLevel.DEBUG)))
                for (Iterator it = c.iterator(); it.hasNext();)
                    logger.log(BasicLevel.DEBUG, "source file: " + it.next());
            ArrayList notFoundMOs = new ArrayList();
            // Scans SpeedoClass objects. If there is no matching Jorm specification,
            // adds the SpeedoClass object into notFoundMOs.
            for (Iterator itDesc = scp.getXmldescriptor().values().iterator(); itDesc.hasNext();) {
                SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
                List scs = desc.getSpeedoClasses();
                for (int i=0; i<scs.size(); i++) {
                    SpeedoClass sc = (SpeedoClass) scs.get(i);
                    String fn = sc.getJormFileName();
                    logger.log(BasicLevel.DEBUG,
                            "looking for the file: " +
                            scp.jormDir + File.separatorChar + fn); //TODO
                    if ((c == null) || (!c.contains(fn))) {         //TODO
                        notFoundMOs.add(sc);
                    }
                }
            }

            // if .pd files are not specified by the user then the jorm meta
            // information is generated
            jormcompiler.setupLogger();
            jormcompiler.setupMIManager();
            Collection createdMOs = null;
            if (!notFoundMOs.isEmpty()) {
                // Turns JDO meta-objects into JORM meta-objects (only those for which
                // there does not exist a JORM specification).
                logger.log(BasicLevel.DEBUG, "JormWriter: projectname: <" + scp.projectName +
                        "> mappername: " + scp.mapperName);
                createdMOs = new JormMIBuilder(
                        jormcompiler.getMIManager(), scp.nmf, logger)
                        .createMI(notFoundMOs, scp.projectName, scp.mapperName);
            }

            if (c != null) {       //TODO
                // Parse the .pd files
                // Builds JORM meta-objects from the collection of .pd files and
                // compares JDO meta-objects with the matching JORM meta-objects.
                c = new ArrayList(jormcompiler.parseFiles(c));
                //isCompatible();
            }
            // Generates JORM files from the JORM meta-objects that have been
            // built from JDO meta-objects.
            if (createdMOs != null && !createdMOs.isEmpty()) {
                jormcompiler.generateJormFiles(createdMOs);
            }
            if (c != null) {   // TODO
                c.addAll(createdMOs);
            }
            else {
                c = createdMOs;
            }

            if (logger.isLoggable(BasicLevel.DEBUG))
                for (Iterator it = c.iterator(); it.hasNext();) {
                    Object o = it.next();
                    if (o instanceof Class) {
                        logger.log(BasicLevel.DEBUG, "Class : "
                                + ((Class) o).getFQName());
                    } else if (o instanceof CompositeName) {
                        logger.log(BasicLevel.DEBUG, "compositeName : "
                                + ((CompositeName) o).getFQName());
                    } else
                        logger.log(BasicLevel.DEBUG, "metaObject : " + o);
                }

            //Verify the meta information
            //jormcompiler.verifyMetaInfo(c);
        } catch (PException e) {
            throw new SpeedoException("Error during .pd files generation", e);
        }
    }

    /**
     * Verifies that all persistent capable field for jdo is described in JORM.
     *
     * @exception org.objectweb.speedo.generation.api.SpeedoXMLError if JORM an JDO MetaData are not coherent
     */
    protected void isCompatible() throws SpeedoException {

        List except = new ArrayList();

        Manager manager = jormcompiler.getMIManager();
        for (Iterator itDesc = scp.getXmldescriptor().values().iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            List scs = desc.getSpeedoClasses();
            for (int i=0; i<scs.size(); i++) {
                compareClass((SpeedoClass) scs.get(i), manager, except);
            }
        }
        if (!except.isEmpty() && logger.isLoggable(BasicLevel.ERROR))
            for (Iterator it = except.iterator(); it.hasNext();)
                logger.log(BasicLevel.ERROR, it.next());
    }

    protected void compareClass(SpeedoClass clas,
                                Manager manager,
                                List except) throws SpeedoException {
        debug = logger.isLoggable(BasicLevel.DEBUG);
        if (debug) {
            logger.log(BasicLevel.DEBUG,
                    "Verify the definition of the class " + clas.getFQName());
        }
        Class classJorm = manager.getClass(clas.getFQName());
        if (classJorm == null) {
            throw new SpeedoException("Jorm description of the class '"
                    + clas.getFQName() + "' is not availlable");
        }
        NameDef pName = null;
	    SpeedoClass sc = clas;
	    while(sc.superClassName != null) {
		    SpeedoClass _sc = sc;
	        sc = sc.jdoPackage.jdoXMLDescriptor.smi.getSpeedoClass(sc.superClassName, sc.jdoPackage);
		    if (sc == null) {
			    throw new SpeedoException("Class " + _sc.getFQName() + " not defined in the jorm meta information");
		    }
	    }
	    Class pclassJorm = manager.getClass(sc.getFQName());
        pName = getClassNameDef(pclassJorm);

        if (classJorm == null) {
			throw new SpeedoXMLError("Class '" + clas.name + "' not defined in JORM metadata");
        }

        // Comparison from JDO
        clas.jormclass = classJorm;
        for (Iterator efield = clas.jdoField.values().iterator(); efield.hasNext();) {
            SpeedoField field = (SpeedoField) efield.next();
            String fieldName = field.name;

            // Fields with this persistent modifier shouldn't stay in JDO Metadata
            if (field.persistenceModifier == SpeedoModifier.none)
                continue;

            TypedElement tElem = classJorm.getTypedElement(fieldName);
            if (tElem == null) {
				throw new SpeedoXMLError("Field '" + fieldName
				    + "' not defined in JORM metadata of the class '"
				    + clas.getFQName() + "'");
            }

            // for application identity key fields have to be compared
            if (field.primaryKey) {
                boolean found = false;
                for (Iterator it = pName.iterateField(); it.hasNext() && !found;) {
                    found = fieldName.equals(it.next());
                }
                if (!found) {
                    throw new SpeedoXMLError("Field '" + fieldName
                        + "' not defined in Class NameDef's Fields of the class '"
                        + clas.getFQName() + "'");
                }
            }

            if (tElem instanceof GenClassRef && field.jdoTuple == null)
				throw new SpeedoXMLError("field '" + fieldName + "' should be a tuple in JDO metadata of the class '" + clas.getFQName() + "'");
        }

        // Comparison from JORM
        for (Iterator jormfield = classJorm.getFields().iterator(); jormfield.hasNext();) {
            TypedElement te = (TypedElement) jormfield.next();
            String fieldName = te.getName();
            if (!clas.jdoField.containsKey(fieldName)
                    && !isContainerIdField(classJorm, te, clas))
                throw new SpeedoXMLError("Field '" + fieldName + "' of the class '"
                            + clas.getFQName() + "' is not defined in the '"
                            + clas.jdoPackage.jdoXMLDescriptor.xmlFile
                            + "' file (found: " + clas.jdoField.keySet() + ").");
        }

        // for application identity key fields have to be compared
        if (clas.identityType == SpeedoIdentity.USER_ID) {
            boolean found = false;
            for (Iterator it = pName.iterateField(); it.hasNext() && !found;) {
                String fn = (String) it.next();
                SpeedoField sf = (SpeedoField) clas.jdoField.get(fn);
                if (sf == null) {
                    logger.log(BasicLevel.WARN, "Field " + fn + " defined in the identifier of "
                            + clas.name
                            + ".pd file but not availlable in the class (or .pd file).");
                } else if (!sf.primaryKey) {
                    except.add("Field " + fn + " defined in the "
                            + clas.name
                            + ".pd file but is not marked as a primary key field in the "
                            + clas.jdoPackage.jdoXMLDescriptor.xmlFile
                            + " file.");
                }
            }
        }
    }

    private boolean isContainerIdField(Class clazz,
                                       TypedElement te,
                                       SpeedoClass sc) throws SpeedoException {
        return sc.identityType == SpeedoIdentity.CONTAINER_ID
                && getClassNameDef(clazz).getNameRef()
                .getProjection().containsValue(te.getName());
    }


    private Mapping getMapping(Class clazz) throws SpeedoException {
        ClassProject cp = clazz.getClassProject(scp.projectName);
        if (cp == null) {
            throw new SpeedoException("No classproject found for the class "
                    + clazz.getFQName() + " and  the project " + scp.projectName);
        }
        int idx =  scp.mapperName.indexOf('.');
        Mapping m = cp.getMapping(idx == -1
                ? scp.mapperName
                : scp.mapperName.substring(0, idx));
        if (m == null) {
            throw new SpeedoException("No mapping found for the class "
                    + clazz.getFQName() + ",  the project " + scp.projectName
                    + " and  the mapper " + scp.mapperName);
        }
        return m;
    }

    private NameDef getClassNameDef(Class clazz) throws SpeedoException {
        return (NameDef) getMapping(clazz).getClassMapping()
                .getIdentifierMapping().getLinkedMO();
    }

    private void addMOClass(Class mo, Set mos) {
        mos.add(mo);
        for(Iterator it = mo.getSuperClasses().iterator();it.hasNext();) {
            addMOClass((Class) it.next(), mos);
        }
    }
}
