/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.workingset.api;

import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.TransactionalWorkingSet;
import org.objectweb.perseus.persistence.api.ConnectionHolder;

import javax.jdo.JDOFatalException;

/**
 * This interfaces defines the transaction concept in Speedo. It is based on a
 * working set represented by the inheritance of the TransactionalWorkingSet
 * interface, and the required javax.jdo.Transaction interface.
 *
 * @author S.Chassande-Barrioz
 */
public interface Transaction
        extends javax.jdo.Transaction, TransactionalWorkingSet {

    /**
     * It activates the working set. This is used to delimit the begining of
     * the working set.
     */
    void active() throws PersistenceException;

    /**
     * Indicates if the transaction environnement is managed. If the 'false'
     * value is returned then that means that a user transaction matches to
     * a local transaction managed by speedo.
     */ 
    boolean isManagedEnv();

	JDOFatalException rollBackOnInternalError(Exception e);

	/**
	 * Assignes the connection holder. The connection holder is able to allocate
	 * a connection to the underlying data support and to kept it until the
	 * working set / transaction end.
	 */
	void setConnectionHolder(ConnectionHolder ch);
}
