/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.workingset.lib;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.WorkingSet;
import org.objectweb.perseus.persistence.api.WorkingSetManager;
import org.objectweb.util.monolog.api.Logger;

/**
 * The class is an implementation of the WorkingSetManager interface. This
 * implementation is empty and do nothing. Indeed a WorkingSet is a fractal
 * component linked to a ProxyManager instance. The the WorkingSet instances are
 * pooled in same time that the ProxyManager instance. Then Speedo never asks to
 * the TransactionalPersistenceManager (Perseus) to create WorkingSet instances.
 *
 * @author S.Chassande-Barrioz
 */
public class SpeedoWorkingSetManager
		implements WorkingSetManager, BindingController {

	private Logger logger;

	// IMPLEMENTATION OF THE UserBindingController INTERFACE //
	//-------------------------------------------------------//

    public String[] listFc() {
        return new String[] {};
    }

	public Object lookupFc(String c) {
		return null;
	}

	public void bindFc(String c, Object s) {
		if ("logger".equals(c)) {
			logger = (Logger) s;
		}
	}

	public void unbindFc(String c) {
	}


	public WorkingSet createWS(Object userObject)
			throws PersistenceException {
		return userObject instanceof WorkingSet ? (WorkingSet) userObject : null;
	}

	public WorkingSet createWS(Object userObject,
							   Object workingSetType)
			throws PersistenceException {
		return userObject instanceof WorkingSet ? (WorkingSet) userObject : null;
	}

	public void closeWS(WorkingSet ws) {
	}
}
