/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.pobjects.basic;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.jdo.InstanceCallbacks;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Query;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class BasicA implements Serializable, InstanceCallbacks {

    public final static int CALLBACK_INIT = 1;
	static String nonPersistentField1 = "static field not persistent";
	final static String nonPersistentField2 = "final static field not persistent";
    public String f1 = null;
	public int f2 = 0;
    private int privateField = -1;
    protected int protectedField = -2;
    int noModifierField = -3;
    private Date d;
    private Date d2;
    private long l;
    protected String undeclaredField;
    private String nonullField = "nonnull";
	private char[] myArrayOfchar;
	private int checkJdoLoad = CALLBACK_INIT;
	private int checkJdoStore = CALLBACK_INIT;
	private int checkJdoClear = CALLBACK_INIT;
	private int checkJdoDelete = CALLBACK_INIT;
	private Locale locale = Locale.getDefault();

	public static List getAllBasicA(PersistenceManagerFactory pmf) {
		PersistenceManager pm = pmf.getPersistenceManager();
		Query q = pm.newQuery();
		q.setClass(BasicA.class);
		Collection col = (Collection) q.execute();
		ArrayList al = new ArrayList();
		for(Iterator it= col.iterator(); it.hasNext();) {
			al.add(it.next());
		}
		q.closeAll();
		pm.close();
		return al;
	}


	private BasicA(String f1, int f2) {
		this.f1 = f1;
		if (this.f1 != null) {
			myArrayOfchar = this.f1.toCharArray();
		} else {
			myArrayOfchar = null;
		}
		this.f2 = f2;
        d = getTime();
        l = 1234567890L;
        undeclaredField = f1 + "_undeclared";
	}

	protected BasicA(String f1) {
		this();
		if (d == null) {
			throw new NullPointerException("d is null");
		}
		this.f1 = f1;
		if (this.f1 != null) {
			myArrayOfchar = f1.toCharArray();
		} else {
			myArrayOfchar = null;
		}
        d = getTime();
        l = 1234567890L;
        undeclaredField = f1 + "_undeclared";
	}
	public BasicA() {
        d = getTime();
        l = 1234567890L;
        undeclaredField = "undeclared";
	}

    public Date getTime() {
        return new Date((Calendar.getInstance().getTime().getTime() / 1000) * 1000);
    }

	public String readF1() {
		return f1;
	}

	public void writeF1(String f1) {
		this.f1 = f1;
		if (this.f1 != null) {
			myArrayOfchar = f1.toCharArray();
		} else {
			myArrayOfchar = null;
		}
	}

	public int readF2() {
		return f2;
	}

	public void writeF2(int f2) {
		this.f2 = f2;
	}

	public void incF2() {
		this.f2 = this.f2 + 1;
	}

	public String readF1_F2() {
		return f1 + f2;
	}

	public void writeF1() {
		f1 = "azerty";
		undeclaredField = f1.toString();
		myArrayOfchar = this.f1.toCharArray();
	}

	public void writeF2() {
		f2 = 1;
	}

	public void writeF1_F2() {
		f1 = "qsdfgh";
		f2 = 2;
	}

	public void delegateWriteF1() {
		writeF1();
	}

	public void jdoPostLoad() {
		checkJdoLoad++;
		readF1_F2();
	}

	public void jdoPreStore() {
		checkJdoStore++;
	}

	public void jdoPreClear() {
		checkJdoClear++;
	}

	public void jdoPreDelete() {
		checkJdoDelete++;
	}

	public int getCheckJdoLoad() {
		return checkJdoLoad;
	}

	public int getCheckJdoStore() {
		return checkJdoStore;
	}

	public int getCheckJdoClear() {
		return checkJdoClear;
	}

	public int getCheckJdoDelete() {
		return checkJdoDelete;
	}

	public Locale getLocale() {
		return locale;
	}

	public void setLocale(Locale l) {
		this.locale = l;
	}

	public String getNonullField() {
		return nonullField;
	}

	public void setNonullField(String nonullField) {
		this.nonullField = nonullField;
	}
	
    public String getUndeclaredField() {
        return undeclaredField;
    }
    public void setUndeclaredField(String undeclaredField) {
        this.undeclaredField = undeclaredField;
    }
}
