/*
 * Created on 29 oct. 2004
 */
package org.objectweb.speedo.pobjects.cap;

import java.util.Collection;

/**
 * @author mhoudu
 */
public class JDORole {

    /** the role identifier */
    private int id;
    
    /** the role label */
    private String label;
    
    /** the role scopes */
    private Collection scopes;
    
    /** the role actions type JDORoleHasAction*/
    private Collection actions;
    
    /**
     * Default constructor.
     * @param id The role identifier.
     * @param label The role label.
     * @param scopes The role scopes.
     * @param actions The role actions.
     */
    public JDORole(
            int id,
            String label,
            Collection scopes,
    		Collection actions)
    {
        this.id = id;
        this.label = label;
        this.scopes = scopes;
        this.actions = actions;
    }

    /**
     * Adds an action to the role.
     * @param action The action to add.
     * @param isDisplayedInMenu The flag which indicates if the action has to be displayed in the menu or not.
     * @param displayOrder The isplay order.
     */
    public void addAction(
            JDOAction action,
            boolean isDisplayedInMenu,
            int displayOrder)
    {
        JDORoleHasAction action4role = new JDORoleHasAction(this, action, isDisplayedInMenu, displayOrder);
        actions.add(action4role);
    }
    
    /**
     * Tells if this role has the specified scope.
     * @param scope the scope to check.
     * @return <tt>true</tt> if this role has the specified scope,
     * <tt>false</tt> otherwise.
     */
    public boolean hasScope(JDOScope scope)
    {
        return scopes.contains(scope);
    }

    /**
     * Tells if this role has the specified action.
     * @param action the action to check.
     * @return <tt>true</tt> if this role has the specified action,
     * <tt>false</tt> otherwise.
     */
    public boolean hasAction(JDOAction action)
    {
        return actions.contains(action);
    }

    /**
     * Gets the id.
     * @return Returns the id.
     */
    public int getId()
    {
        return id;
    }
    
    /**
     * Sets the id.
     * @param id The id to set.
     */
    public void setId(int id)
    {
        this.id = id;
    }
    
    /**
     * Gets the label.
     * @return Returns the label.
     */
    public String getLabel()
    {
        return label;
    }
    
    /**
     * Sets the label.
     * @param label The label to set.
     */
    public void setLabel(String label)
    {
        this.label = label;
    }
    
    /**
     * Gets the scopes.
     * @return Returns the scopes.
     */
    public Collection getScopes()
    {
        return scopes;
    }
    
    /**
     * Sets the scopes.
     * @param scopes The scope to set.
     */
    public void setScopes(Collection scopes)
    {
        this.scopes = scopes;
    }

    /**
     * Gets the actions.
     * @return Returns the actions.
     */
    public Collection getActions()
    {
        return actions;
    }
    
    /**
     * Sets the actions.
     * @param actions The actions to set.
     */
    public void setActions(Collection actions)
    {
        this.actions = actions;
    }
   
}