/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.pobjects.detach;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * @author Y.Bersihand
 */
public class SharePrice implements Serializable {

	  private long id;          // price object identification
	  private int type;         // the type of the price
	  private int year;         // the year when this price is valid for
	  private int month;        // the month when this price is valid for
	  private BigDecimal price; // The price valid for this type, year and month

	  private Share share;

	  public SharePrice() {
	  }

	  public SharePrice(int aType, int aYear, int aMonth, BigDecimal aPrice) {
	  	type = aType;
		year = aYear;
		month = aMonth;
		price = aPrice;
	  }

	  public long getId() {
	  	return id;
	  }

	  public void setId(long aId) {
	  	id = aId;
	  }

	  public Share getShare() {
	  	return share;
	  }

	  public void setShare(Share aShare) {
	  	share = aShare;
	  }

	  public int getType() {
	  	return type;
	  }

	  public void setType(int type) {
	  	this.type = type;
	  }

	  public int getYear() {
	  	return year;
	  }

	  public void setYear(int year) {
	  	this.year = year;
	  }

	  public int getMonth() {
	  	return month;
	  }

	  public void setMonth(int month) {
	  	this.month = month;
	  }
		  
	  public BigDecimal getPrice() {
	  	return price;
	  }

	  public String getFormattedPrice() {
	  	return price.toString();
	  }

	  public void setPrice(BigDecimal aPrice) {
	  	price = aPrice;
	  }

	  public boolean equals(Object aObject) {
	  	return aObject instanceof SharePrice && ((SharePrice) aObject).getId() == this.getId();
	  }

	  public int hashCode() {
	  	return new Long(id).hashCode();
	  }
	  
}
