/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.pobjects.inheritance.prefetch;

import java.util.Collection;
import java.util.Iterator;


/**
 * @author Y.Bersihand
 */
public class Worker extends Employee {

	private boolean partTime;
	private Manager mainManager;
	private Department department;
	private Collection days;
	
	public Worker(String name, int age, String company, boolean partTime, Department dpt) {
		super(name, age, company, "worker");
		this.partTime = partTime;
		this.department = dpt;
	}
	
	public boolean isPartTime() {
		return partTime;
	}
	
	public void setPartTime(boolean partTime) {
		this.partTime = partTime;
	}
	
	public Manager getMainManager() {
		return mainManager;
	}
	public void setMainManager(Manager mainManager) {
		this.mainManager = mainManager;
		mainManager.addWorker(this);
	}
		
	public String toString(){
		String s = "Worker: Employee(" + super.toString() + "), " + (partTime?"part-time":"full time");
		if (mainManager != null)
			s += ", main manager=" + mainManager.getName();
		else
			s += ", main manager=NULL";
		if (department != null)
			s += ", " + department.toString();
		if (days != null) {
			s += ", days=[";
			Iterator it = days.iterator();
			while(it.hasNext()){
				Day d = (Day) it.next();
				s += d.toString() + " ";
			}
			s += "]";
		}
		return s;
	}
	
	public Department getDepartment() {
		return department;
	}
	public void setDepartment(Department department) {
		this.department = department;
	}
	public Collection getDays() {
		return days;
	}
	public void setDays(Collection days) {
		this.days = days;
	}
}
