/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.inheritance;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.Extent;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.inheritance.filterOutOfPK.composite.A1;
import org.objectweb.speedo.pobjects.inheritance.filterOutOfPK.composite.A21;
import org.objectweb.speedo.pobjects.inheritance.filterOutOfPK.composite.A22;
import org.objectweb.speedo.pobjects.inheritance.filterOutOfPK.composite.A3;
import org.objectweb.speedo.pobjects.inheritance.filterOutOfPK.composite.B;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Test the inheritance with a single user id and a filter out of the id.
 * @author Y.Bersihand
 */
public class TestCompositeFilterOutOfPK extends SpeedoTestHelper {
	
	public TestCompositeFilterOutOfPK(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.inheritance.TestCompositeFilterOutOfPK";
	}
	
	int nbOfA1 = 3;
	int nbOfA22 = 2;
	int nbOfA3 = 2;
	int nbToRetrieve = 0;
	
	public void testCompositeFilterOutOfPK() {
		logger.log(BasicLevel.DEBUG, "***************testCompositeFilterOutOfPK*****************");
		//create the forms
		createInheritedObjects();
		//get all the A1
		iterateExtent(A1.class);
		//get all the A21 
		iterateExtent(A21.class);
		//get all the A22 
		iterateExtent(A22.class);
		//get all the A3
		iterateExtent(A3.class);
		//query
		retrieveAllXXA(1000);
	}
	
	/**
	 * Create persistent objects
	 */
	public void createInheritedObjects(){
		PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Collection col = new ArrayList();
			
			B b1 = new B(1000, "1000");
			col.add(b1);
			B b2 = new B(2000, "2000");
			col.add(b2);
			//create A1
			A1[] a1 = new A1[nbOfA1];
			for (int i = 0; i < a1.length; i++) {
				a1[i] = new A1(i+1, ""+(i+1), 1, b1);
				col.add(a1[i]);
				nbToRetrieve++;
			}
			//create A22
			A22[] a22 = new A22[nbOfA22];
			for (int i = 0; i < a22.length; i++) {
				a22[i] = new A22(i+1+nbOfA1, ""+(i+1+nbOfA1), 22, b1, "this is an A22");
				col.add(a22[i]);
				nbToRetrieve++;
			}
			//create A3
			A3[] a3 = new A3[nbOfA3];
			for (int i = 0; i < a3.length; i++) {
				a3[i] = new A3(i+1+nbOfA1+nbOfA22, ""+(i+1+nbOfA1+nbOfA22), 3, b2, true, 9);
				col.add(a3[i]);
			}
			
			
			//	make persistent all the forms
			pm.currentTransaction().begin();
			pm.makePersistentAll(col);
			pm.currentTransaction().commit();
		} catch (Exception e) {
			fail(e.getMessage());
		} finally {
			if (pm.currentTransaction().isActive())
				pm.currentTransaction().rollback();
			//remove all the objects from the cache
			pm.evictAll();
			pm.close();
		}
	}

	//iterate over all the instances of a class
	public void iterateExtent(Class cl){
		PersistenceManager pm = pmf.getPersistenceManager();
		try {
			int nb = 0;
			int nbExpected = 0;
			if (cl == A1.class) {
				nbExpected = nbOfA1 + nbOfA22+ nbOfA3;
			} else if (cl == A21.class) {
				nbExpected = nbOfA22;
			} else if (cl == A22.class) {
				nbExpected = nbOfA22;
			} else if (cl == A3.class) {
				nbExpected = nbOfA3;
			}
			
			pm.currentTransaction().begin();
			Extent extent = pm.getExtent(cl, true);
			Iterator it = extent.iterator();
			String className = cl.getName().substring(cl.getName().lastIndexOf("."));
			logger.log(BasicLevel.DEBUG, "All " + cl.getName() + " instances:");
			while(it.hasNext()){
				A1 a = (A1) it.next();
				assertNotNull("The A should not be null", a);
				nb++;
			}
			extent.close(it);
			pm.currentTransaction().commit();
			assertEquals("The size of elements retrieved is not right.", nbExpected, nb);
		} catch (Exception e) {
			fail(e.getMessage());
		} finally {
			if (pm.currentTransaction().isActive())
				pm.currentTransaction().rollback();
			pm.evictAll();
			pm.close();
		}
	}
	
	public void retrieveAllXXA(int bid){
		PersistenceManager pm = pmf.getPersistenceManager();
		try {
			Query query = pm.newQuery(A1.class,"b.b1==" + bid);
			Collection results = (Collection)query.execute();
			assertEquals(nbToRetrieve, results.size());
			logger.log(BasicLevel.DEBUG, "All A1 linked to the " + bid + " B:");
			Iterator it = results.iterator();
			while(it.hasNext()){
				A1 a = (A1) it.next();
				assertEquals(bid, a.getB().getB1());
			}
			query.closeAll();
		} catch (Exception e) {
			fail(e.getMessage());
		} finally {
			pm.evictAll();
			pm.close();
		}
	}
	public void testRemovingOfPersistentObject() {
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class[] cs = new Class[]{A1.class, B.class};
	        pm.currentTransaction().begin();
            for(int i=0; i<cs.length; i++) {
                Query query = pm.newQuery(cs[i]);
                Collection col = (Collection) query.execute();
                Iterator it = col.iterator();
                while(it.hasNext()) {
                    Object o = it.next();
                    Assert.assertNotNull("null object in the query result"
                        + cs[i].getName(), o);
                    pm.deletePersistent(o);

                }
                query.close(col);
            }
	        pm.currentTransaction().commit();
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }
}
