/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.inheritance;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.Extent;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.inheritance.prefetch.AbstractAssistant;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Day;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Department;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Employee;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Manager;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Secretary;
import org.objectweb.speedo.pobjects.inheritance.prefetch.Worker;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * 
 * @author Y.Bersihand
 */
public class TestPrefetchQueries extends SpeedoTestHelper {
	
	public TestPrefetchQueries(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.inheritance.TestPrefetchQueries";
	}
	
	static final int AGE_LIMIT = 25;
	protected int nbOverLimit;
	
	/**This steps describes how to:
	 * make persistent inherited objects
	 * 
	 */
	public void testInheritance() {
		logger.log(BasicLevel.DEBUG, "***************testInheritance*****************");
		PersistenceManager pm = pmf.getPersistenceManager();
		//create the employees, workers and managers
		createInheritedObjects(pm);
		//remove all the objects from the cache
		pm.evictAll();
		countAllEmployees(pm);
		//get all the employees
		/*iterateExtent(pm, Employee.class);
		//get all the workers
		iterateExtent(pm, Worker.class);
		//get all the managers
		iterateExtent(pm, Manager.class);
		//get all the abstract assistants
		iterateExtent(pm, AbstractAssistant.class);
		//get all the secretaries
		iterateExtent(pm, Secretary.class);
		retrieveOverXX(pm, AGE_LIMIT);
		retrieveOverXXWorker(pm, AGE_LIMIT);
		retrieveOverXXManager(pm, AGE_LIMIT);
		retrieveOverXXSecretary(pm, AGE_LIMIT);*/
        pm.close();
	}
	
	/**
	 * Create persistent objects
	 */
	public void createInheritedObjects(PersistenceManager pm){
		
		Employee employee1 = new Employee("Herve Landry", 42.4, "Tetra Pack");
		Employee employee2 = new Employee("Vincent Racado", 59, "Tetra Pack");
		
		Department dept = new Department("Production");
		
		Worker worker1 = new Worker("Caroline Bret", 33, "Tetra Pack", true, dept);
		Worker worker2 = new Worker("Evelyne Jain", 54, "Tetra Pack", false, dept);
		Worker worker3 = new Worker("Tim Jorge", 28, "Tetra Pack", false, dept);
		
		Collection days = new ArrayList();
		Day day1 = new Day("monday", 1);
		Day day2 = new Day("tuesday", 2);
		Day day3 = new Day("wednesday", 3);
		Day day4 = new Day("thursday", 4);
		Day day5 = new Day("friday", 5);
		days.add(day1);
		days.add(day2);
		days.add(day3);
		days.add(day4);
		days.add(day5);
		
		worker1.setDays(days);
		worker3.setDays(days);
		Manager manager1 = new Manager("Jean Duverge", 57, "Tetra Pack", "Sales");
		Manager manager2 = new Manager("Eric Mento", 49, "Tetra Pack", "Marketing");
		
		worker1.setMainManager(manager1);
		worker2.setMainManager(manager1);
		worker3.setMainManager(manager2);
		
		Secretary sec1 = new Secretary("Durand Luc", 27, "Tetra Pack", 12);
		Secretary sec2 = new Secretary("Serve Amandine", 34, "Tetra Pack", 10);
		
		Collection employees = new ArrayList();
		employees.add(employee1);
		employees.add(employee2);
		employees.add(worker1);
		employees.add(worker2);
		employees.add(worker3);
		employees.add(manager1);
		employees.add(manager2);
		employees.add(sec1);
		employees.add(sec2);
		
		Iterator it = employees.iterator();
		nbOverLimit = 0;
		while(it.hasNext()){
			Employee e = (Employee) it.next();
			if(e.getAge() > AGE_LIMIT)
				nbOverLimit++;
		}
		
		//make persistent all the persons using a collection
		pm.currentTransaction().begin();
        pm.makePersistentAll(employees);
        pm.currentTransaction().commit();
	}

	//iterate over all the instances of a class
	public void iterateExtent(PersistenceManager pm, Class cl){
		Extent extent = pm.getExtent(cl, true);
        Iterator it = extent.iterator();
        String className = cl.getName().substring(cl.getName().lastIndexOf("."));
        logger.log(BasicLevel.DEBUG, "All " + cl.getName() + " instances:");
        while(it.hasNext()){
        	Employee e = (Employee) it.next();
        	assertNotNull(e);
        	logger.log(BasicLevel.DEBUG, e.toString());
        }
        extent.close(it);
	}
	
	//retrieve all the over-XX 
	public void retrieveOverXX(PersistenceManager pm, int limit){
		Query query = pm.newQuery(Employee.class,"age > " + limit);
        query.setOrdering("age ascending");
        Collection results = (Collection)query.execute();
        assertEquals(nbOverLimit, results.size());
        logger.log(BasicLevel.DEBUG, "Over-" + limit + " ordered:");
		Iterator it = results.iterator();
		double age = 0;
		while(it.hasNext()){
			Employee e = (Employee) it.next();
			assertTrue(age < e.getAge());
			age = e.getAge();
			logger.log(BasicLevel.DEBUG, e.toString());
		}
        query.closeAll();
	}
	
	//retrieve all the over-XX worker 
	public void retrieveOverXXWorker(PersistenceManager pm, int limit){
		Query query = pm.newQuery(Worker.class,"age > " + limit);
        query.setOrdering("age ascending");
        Collection results = (Collection)query.execute();
        logger.log(BasicLevel.DEBUG, "Over-" + limit + " ordered:");
		Iterator it = results.iterator();
		double age = 0;
		while(it.hasNext()){
			Employee e = (Employee) it.next();
			assertTrue(age < e.getAge());
			age = e.getAge();
			logger.log(BasicLevel.DEBUG, e.toString());
		}
        query.closeAll();
	}
	
	//retrieve all the over-XX manager 
	public void retrieveOverXXManager(PersistenceManager pm, int limit){
		Query query = pm.newQuery(Manager.class,"age > " + limit);
        query.setOrdering("age ascending");
        Collection results = (Collection)query.execute();
        logger.log(BasicLevel.DEBUG, "Over-" + limit + " ordered:");
		Iterator it = results.iterator();
		double age = 0;
		while(it.hasNext()){
			Employee e = (Employee) it.next();
			assertTrue(age < e.getAge());
			age = e.getAge();
			logger.log(BasicLevel.DEBUG, e.toString());
		}
        query.closeAll();
	}
	
	//retrieve all the over-XX secretary 
	public void retrieveOverXXSecretary(PersistenceManager pm, int limit){
		Query query = pm.newQuery(Secretary.class,"age > " + limit);
        query.setOrdering("age ascending");
        Collection results = (Collection)query.execute();
        logger.log(BasicLevel.DEBUG, "Over-" + limit + " ordered:");
		Iterator it = results.iterator();
		double age = 0;
		while(it.hasNext()){
			Employee e = (Employee) it.next();
			assertTrue(age < e.getAge());
			age = e.getAge();
			logger.log(BasicLevel.DEBUG, e.toString());
		}
        query.closeAll();
	}
	
	public static class CountResult {
		public String name;
		public String type;
		public Integer count;
		
		public CountResult(String name, String type, Integer count) {
			this.name = name;
			this.type = type;
			this.count = count;
		}
		
		 public String toString() {
            return "name: " + name + ", type: " + type + ", count: " + count;
        } 
	}
	
	//	count all employees
	public void countAllEmployees(PersistenceManager pm){
		Query query = pm.newQuery(Employee.class);
		query.setResult("name, type, count(this)");
        query.setGrouping("name, type"); 
		query.setResultClass(CountResult.class);
        Collection results = (Collection)query.execute();
        logger.log(BasicLevel.DEBUG, "Count:");
		Iterator it = results.iterator();
		while(it.hasNext()){
			CountResult cr = (CountResult) it.next();
			logger.log(BasicLevel.DEBUG, "" + cr.toString());
		}
        query.closeAll();
	}
	
	public void testRemovingOfPersistentObject() {
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class[] cs = new Class[]{Employee.class,Day.class,Department.class};
	        pm.currentTransaction().begin();
            for(int i=0; i<cs.length; i++) {
                Query query = pm.newQuery(cs[i]);
                Collection col = (Collection) query.execute();
                Iterator it = col.iterator();
                while(it.hasNext()) {
                    Object o = it.next();
                    Assert.assertNotNull("null object in the query result"
                        + cs[i].getName(), o);
                    pm.deletePersistent(o);

                }
                query.close(col);
            }
	        pm.currentTransaction().commit();
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }
}
