/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.sequence.id.inheritance;

import java.util.Collection;
import java.util.Iterator;

import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.jdo.datastore.Sequence;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.sequence.id.inheritance.Article;
import org.objectweb.speedo.pobjects.sequence.id.inheritance.Materiel;
import org.objectweb.speedo.pobjects.sequence.id.inheritance.Service;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author Y.Bersihand
 */
public class TestSequenceInheritance extends SpeedoTestHelper {
	
	public static final int ADDITIONAL = 5;
	
	public TestSequenceInheritance(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.sequence.TestSequence";
	}
	
	public static final String ARTICLE_SEQ = "org.objectweb.speedo.pobjects.sequence.id.inheritance.article_seq";
	
	/**
	 * Get a sequence and use it to make objects persistent.
	 * A field has a value-strategy set to sequence.
	 */
	public void testSequenceInheritance1() {
		logger.log(BasicLevel.DEBUG, "***************testSequenceInheritance1*****************");
		PersistenceManager pm = pmf.getPersistenceManager();
		try {
			pm.getObjectIdClass(Article.class);
			//	get the sequence
			Sequence s = pm.getSequence(ARTICLE_SEQ);
			assertNotNull("Sequence " + ARTICLE_SEQ + " should not be null.", s);
			s.allocate(ADDITIONAL);
			Article articles[] = new Article[ADDITIONAL];
			for (int i = 0; i < ADDITIONAL; i++) {
				if (i%2 == 0) {
					articles[i] = new Service(); 
				} else {
					articles[i] = new Materiel();
				}
			}
			//	make persistent
			pm.currentTransaction().begin();
			pm.makePersistentAll(articles);
			pm.currentTransaction().commit();
			for (int i = 0; i < ADDITIONAL-1; i++) {
				assertTrue(articles[i].getId() < articles[i+1].getId());
			}
		} catch (Exception e) {
			fail(e.getMessage());
		} finally {
			if (pm.currentTransaction().isActive())
				pm.currentTransaction().rollback();
			pm.close();
		}
	}
	
	/**
	 * Remove all the persistent instances.
	 */
	public void testRemovingOfPersistentObject() {
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class[] cs = new Class[]{Article.class};
	        pm.currentTransaction().begin();
            for(int i=0; i<cs.length; i++) {
                Query query = pm.newQuery(cs[i]);
                Collection col = (Collection) query.execute();
                Iterator it = col.iterator();
                while(it.hasNext()) {
                    Object o = it.next();
                    Assert.assertNotNull("null object in the query result"
                        + cs[i].getName(), o);
                    pm.deletePersistent(o);

                }
                query.close(col);
            }
	        pm.currentTransaction().commit();
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }
}
