/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.tck;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.basic.BasicA;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;

public class JdoHelperTck extends SpeedoTestHelper {

	public static final int TRANSIENT = 0;
	public static final int PERSISTENT_NEW = 1;
	public static final int PERSISTENT_CLEAN = 2;
	public static final int PERSISTENT_DIRTY = 3;
	public static final int HOLLOW = 4;
	public static final int TRANSIENT_CLEAN = 5;
	public static final int TRANSIENT_DIRTY = 6;
	public static final int PERSISTENT_NEW_DELETED = 7;
	public static final int PERSISTENT_DELETED = 8;
	public static final int PERSISTENT_NONTRANSACTIONAL = 9;
	public static final int NUM_STATES = 10;
	public static final int ILLEGAL_STATE = 10;

	private static final int IS_PERSISTENT = 0;
	private static final int IS_TRANSACTIONAL = 1;
	private static final int IS_DIRTY = 2;
	private static final int IS_NEW = 3;
	private static final int IS_DELETED = 4;
	private static final int NUM_STATUSES = 5;

	private static final boolean state_statuses[][] = {
		//      IS_PERSISTENT IS_TRANSACTIONAL IS_DIRTY IS_NEW IS_DELETED
		// transient
		{ false, false, false, false, false },

		// persistent-new
		{
			true, true, true, true, false },

		// persistent-clean
		{
			true, true, false, false, false },

		// persistent-dirty
		{
			true, true, true, false, false },

		// hollow
		{
			true, false, false, false, false },

		// transient-clean
		{
			false, true, false, false, false },

		// transient-dirty
		{
			false, true, true, false, false },

		// persistent-new-deleted
		{
			true, true, true, true, true },

		// persistent-deleted
		{
			true, true, true, false, true },

		// persistent-nontransactional
		{
			true, false, false, false, false }
	};

	/**
	 * This utility method returns a <code>String</code> that indicates the current state of an instance.
	 * 
	 * @param o
	 *            The object.
	 * @return The current state of the instance, by using the <code>JDOHelper</code> state interrogation methods.
	 */
	public static String getStateOfInstance(Object o) {
		boolean existingEntries = false;
		StringBuffer buff = new StringBuffer("{");
		if (JDOHelper.isPersistent(o)) {
			buff.append("persistent");
			existingEntries = true;
		}
		if (JDOHelper.isTransactional(o)) {
			if (existingEntries)
				buff.append(", ");
			buff.append("transactional");
			existingEntries = true;
		}
		if (JDOHelper.isDirty(o)) {
			if (existingEntries)
				buff.append(", ");
			buff.append("dirty");
			existingEntries = true;
		}
		if (JDOHelper.isNew(o)) {
			if (existingEntries)
				buff.append(", ");
			buff.append("new");
			existingEntries = true;
		}
		if (JDOHelper.isDeleted(o)) {
			if (existingEntries)
				buff.append(", ");
			buff.append("deleted");
		}
		buff.append("}");
		return buff.toString();
	}

	public static int currentState(Object o) {
		boolean[] status = new boolean[5];
		status[IS_PERSISTENT] = JDOHelper.isPersistent(o);
		status[IS_TRANSACTIONAL] = JDOHelper.isTransactional(o);
		status[IS_DIRTY] = JDOHelper.isDirty(o);
		status[IS_NEW] = JDOHelper.isNew(o);
		status[IS_DELETED] = JDOHelper.isDeleted(o);
		int i, j;
		outerloop : for (i = 0; i < NUM_STATES; ++i) {
			for (j = 0; j < NUM_STATUSES; ++j) {
				if (status[j] != state_statuses[i][j])
					continue outerloop;
			}
			return i;
		}
		return NUM_STATES;
	}

	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction transaction;

	public JdoHelperTck(String s) {
		super(s);
		//pm = pmf.getPersistenceManager();
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.tck.JdoHelperTck";
	}
/*
	public void testGetObjectId() {
		logger.log(BasicLevel.INFO, "testGetObjectId");
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA nba1 = new BasicA();
			pm.makePersistent(nba1);

			Object p2 = JDOHelper.getObjectId(nba1);
			tx.commit();

			Assert.assertNotNull("bad getObjectId value", p2);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testGetObjectId", ex);
			fail(ex.getMessage());
		}

	}

	public void testGetObjectIdForNull() {
		logger.log(BasicLevel.INFO, "testGetObjectIdForNull");
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			Object p1 = null;
			Object oid = JDOHelper.getObjectId(p1);
			tx.commit();

			Assert.assertTrue("bad oid value", oid == null);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testGetObjectIdForNull", ex);
			fail(ex.getMessage());
		}
	}

	public void testGetPersistenceManager() {
		logger.log(BasicLevel.INFO, "testGetPersistenceManager");
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA nba1 = new BasicA();
			pm.makePersistent(nba1);
			pm1 = JDOHelper.getPersistenceManager(nba1);
			tx.commit();
			assertTrue("bad getPersistenceManager value", (pm1 != null));

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testGetPersistenceManager", ex);
			fail(ex.getMessage());
		}
	}

	public void testGetTransactionalObjectId() {
		logger.log(BasicLevel.INFO, "testGetTransactionalObjectId");
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA nba1 = new BasicA();
			pm.makePersistent(nba1);

			Object p2 = JDOHelper.getTransactionalObjectId(nba1);
			tx.commit();

			assertTrue("bad getTransactionalObjectId value", (p2 != null));

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testGetTransactionalObjectId", ex);
			fail(ex.getMessage());
		}

	}

	public void testIsDeletedFalse() {
		logger.log(BasicLevel.INFO, "testIsDeletedFalse");
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA p1 = new BasicA();
			pm.makePersistent(p1);
			//pm.deletePersistent(p1);
			boolean flag = JDOHelper.isDeleted(p1);
			tx.commit();

			assertTrue("bad isDeleted value", !flag);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testIsTransactionalFalse", ex);
			fail(ex.getMessage());
		}

	}

	public void testIsTransactionalFalse() {
		logger.log(BasicLevel.INFO, "testIsTransactionalFalse");

		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA p1 = new BasicA();
			pm.makePersistent(p1);
			boolean flag = JDOHelper.isTransactional(p1);
			tx.commit();

			Transaction tx1 = pm.currentTransaction();
			tx1.begin();
			flag = JDOHelper.isTransactional(p1);
			tx1.commit();

			assertTrue("bad isTransactional value", !flag);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testIsTransactionalFalse", ex);
			fail(ex.getMessage());
		}
	}
*/
	public void testMakeDirty() {
		logger.log(BasicLevel.INFO, "testMakeDirty");
		try {
			pm = pmf.getPersistenceManager();
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA p1 = new BasicA();
			pm.makePersistent(p1);
			tx.commit();

			Transaction tx1 = pm.currentTransaction();
			tx1.begin();
			JDOHelper.makeDirty(p1, "f1");

			int state = currentState(p1);
			logger.log(
				BasicLevel.INFO,
				"currentState(p1)=" + state + "getStateOfInstance(p1)=" + getStateOfInstance(p1));

			boolean flag = JDOHelper.isDirty(p1);
			tx1.commit();

			assertTrue("bad isDirty value", flag);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testMakeDirty", ex);
			fail(ex.getMessage());
		} finally {
			pm.close();
		}
	}

	public void testIsDeletedForTransient() {
		logger.log(BasicLevel.INFO, "testIsDeletedForTransient");
		try {
			pm = pmf.getPersistenceManager();
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA p1 = new BasicA();
			
			int state = currentState(p1);

			logger.log(BasicLevel.INFO, "IS_PERSISTENT=" + JDOHelper.isPersistent(p1));
			logger.log(BasicLevel.INFO, "IS_TRANSACTIONAL=" + JDOHelper.isTransactional(p1));
			logger.log(BasicLevel.INFO, "IS_DIRTY=" + JDOHelper.isDirty(p1));
			logger.log(BasicLevel.INFO, "IS_NEW=" + JDOHelper.isNew(p1));
			logger.log(BasicLevel.INFO, "IS_DELETED=" + JDOHelper.isDeleted(p1));

			logger.log(
				BasicLevel.INFO,
				"currentState(p1)=" + state + " getStateOfInstance(p1)=" + getStateOfInstance(p1));

			boolean flag = JDOHelper.isDeleted(p1);
			tx.commit();

			assertTrue("bad isDeletedforTransient value", !flag);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testIsDeletedForTransient", ex);
			fail(ex.getMessage());
		} finally {
			pm.close();
		}
	}

}
