/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.tck;

import java.util.ArrayList;
import java.util.Collection;

import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.jdo.Transaction;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.basic.BasicA;
import org.objectweb.util.monolog.api.BasicLevel;



public class QueryTck extends SpeedoTestHelper {

	private PersistenceManager          pm;
	private PersistenceManager          pm1;
	private Transaction                 transaction;
	
	
	public QueryTck(String s) {
		super(s);		
		pm = pmf.getPersistenceManager();		
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.tck.QueryTck";
	}

	public void testGetObjectId() {
		logger.log(BasicLevel.INFO, "testGetObjectId");
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			BasicA nba1 = new BasicA();
			pm.makePersistent(nba1);			

			Object p2 = JDOHelper.getObjectId(nba1);
			tx.commit();
			
			Assert.assertNotNull("bad getObjectId value", p2);

		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testGetObjectId", ex);
			fail(ex.getMessage());
		}
		
	}

	public void testDeclareVariables() {
		logger.log(BasicLevel.INFO, "testDeclareVariables");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if(query == null) { 
			logger.log(BasicLevel.ERROR, "query is null");
		}

		query.setClass(BasicA.class);
		query.setCandidates(pm.getExtent(BasicA.class, false));
		query.declareParameters("Integer param1, Integer param2");
		query.setFilter("f1 == param1 && f2 == param2");
		
		Object results = query.execute(new String(""), new java.lang.Integer(0));

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new BasicA();
		expected.add(p3);
/*
		insertExpected(expected);

		

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception ex) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testDeclareVariables", ex);
			fail(ex.getMessage());
		}
*/		
		tx.commit();
		
	}
}

