/*
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 */
package org.objectweb.speedo.j2eedo.bo;

import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import org.objectweb.speedo.Alea;
import org.objectweb.speedo.j2eedo.database.Address;
import org.objectweb.speedo.j2eedo.database.Department;
import org.objectweb.speedo.j2eedo.database.Employee;
import org.objectweb.speedo.j2eedo.database.Project;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * This class define the Java class to methods used to handle employee
 * @author fmillevi@yahoo.com
 *
 */
public class EmployeeFactory {
	static Logger logger = null;
	private StringBuffer outStr;
	private PersistenceManager pm;
	static {
		LoggerFactory lf = Monolog.initialize();
		logger = lf.getLogger(DatabaseImpl.class.getName());
	}


	/**
	 * Initializes the StringBuffer used to store the treatment result
	 * @param outStr The outStr to set.
	 */
	public void setOutStr(StringBuffer outStr) {
		this.outStr = outStr;
	}

	/**
	 * Initializes the JDO persistence manager
	 * @param pm The pm to set.
	 */
	public void setPm(PersistenceManager pm) {
		this.pm = pm;
	}

	/**
	 * Creates a new employee in one of the existing department
	 * @throws JDOException
	 * @throws Exception
	 */
	public void newEmployee(PollsSynchronizations pollsSync) throws JDOException, Exception {
		long id = DatabaseImpl.getDepartmentIdFromPool();
		Department d =
			(Department) pm.getObjectById(
					pm.newObjectIdInstance(Department.class, Long.toString(id)),
					false);
		this.outStr.append("\nCreate a new employee ");
		Calendar cal = Calendar.getInstance();
		cal.set(Alea.rand(1900, 2004), Alea.rand(1, 12), Alea.rand(1, 28));
		Employee e =
			new Employee(
					"First-" + Alea.randomstring(4, 5),
					"Last-" + Alea.randomstring(8, 10),
					cal.getTime(),
					d);
		e.setSalary(Alea.rand(5000, 45000));
		Address a = new Address();
		a.setCity("City-" + Alea.randomstring(4, 5));
		a.setStreet("Street-" + Alea.randomstring(4, 5));
		a.setState("State-" + Alea.randomstring(4, 5));
		a.setZipcode("ZIP-" + Alea.randomstring(2, 3));
		e.setAddress(a);
		pm.makePersistent(e);
		logger.log(BasicLevel.DEBUG, "Create a new employee having id=" + e.getEmpid());

		pollsSync.addInPool(DatabaseImpl.poolOfEmployeeId, e.getEmpid());
		pm.setUserObject( pollsSync);

	}

	/**
	 * Deletes an existing employee
	 * @throws JDOException
	 * @throws Exception
	 */
	public void deleteEmployee(PollsSynchronizations pollsSync) throws JDOException, Exception {
		long id = DatabaseImpl.getEmployeeIdFromPool();
		// remove id from static pool to avoid other threatment to use it
		pollsSync.removeFromPool(DatabaseImpl.poolOfEmployeeId, id);
		pm.setUserObject( pollsSync);
		this.outStr.append("\nRemove the employee ");
		this.outStr.append(id);
		logger.log(BasicLevel.DEBUG, "Delete employee having id=" + id);
		Employee e =
			(Employee) pm.getObjectById(
					pm.newObjectIdInstance(Employee.class, Long.toString(id)),
					false);
		Address a = e.getAddress();
		pm.deletePersistent(a);
		pm.deletePersistent(e);
	}

	/**
	 * Gets employee's data
	 * @throws JDOException
	 * @throws Exception
	 */
	public void getEmployee() throws JDOException, Exception {
		long id = DatabaseImpl.getEmployeeIdFromPool();
		this.outStr.append("\nGet the employe id ");
		this.outStr.append(id);
		logger.log(BasicLevel.DEBUG, "Get employee having id=" + id);
		Employee e =
			(Employee) (Employee) pm.getObjectById(
					pm.newObjectIdInstance(Employee.class, Long.toString(id)),
					false);
		this.outStr.append(e.getAsString());
	}

	/**
	 * Gets some employees. 4 queries are defines
	 * <ul><li>Get an employee by it's primary key</li>
	 * <li>Get employees having it's id between two values</li>
	 * <li>Get employees having the same manager</li>
	 * <li>Get employees member of one of 2 selected projects</li></ul>
	 * @throws JDOException
	 * @throws Exception
	 */
	public void getEmployees() throws JDOException, Exception {
		int testId = Alea.rand(0, 3);
		switch (testId) {
		case 0 :
			this.queryEmployee();
			break;
		case 1 :
			this.queryEmployeeArrayParameters();
			break;
		case 2 :
			this.queryEmployeeByManager();
			break;
		case 3 :
			this.queryEmployeeOrderHiredateByProject();
			break;
		}
	}

	/**
	 * Increases an employee' salary
	 * @throws JDOException
	 * @throws Exception
	 */
	public void increaseSalary() throws JDOException, Exception {
		long id = DatabaseImpl.getEmployeeIdFromPool();
		this.outStr.append("\nIncrease salary for the employe ");
		this.outStr.append(id);
		logger.log(
				BasicLevel.DEBUG,
				"Increase the salary for the employee having id=" + id);
		Employee e =
			(Employee) pm.getObjectById(
					pm.newObjectIdInstance(Employee.class, Long.toString(id)),
					false);
		e.setSalary(e.getSalary() + 1);
	}

	private void queryEmployee() throws JDOException, Exception {
		long id = DatabaseImpl.getEmployeeIdFromPool();
		logger.log(BasicLevel.DEBUG, "Query employee having id=" + id);
		Query query = pm.newQuery(Employee.class);

		String filter = "(empid==aId)";
		String param = "long aId";

		query.declareParameters(param);
		query.setFilter(filter);

		Collection col = (Collection) query.execute(new Long(id));
		Iterator iter = col.iterator();
		if (!iter.hasNext())
			throw new Exception("TestInheritanceBasic query on employee does not return any row");
		Employee e = (Employee) iter.next();
		if (e.getEmpid() != id)
			throw new Exception("TestInheritanceBasic query on employee returns an other row");
		if (iter.hasNext())
			throw new Exception("TestInheritanceBasic query on a employee returns to many rows");
		this.outStr.append("\nEmployee full name ");
		this.outStr.append(e.getFirstname());
		this.outStr.append(" ");
		this.outStr.append(e.getLastname());
		query.close(col);
	}

	private void queryEmployeeOrderHiredateByProject()
	throws JDOException, Exception {
		Project p =
			(Project) pm.getObjectById(
					pm.newObjectIdInstance(
							Project.class,
							Long.toString(
									DatabaseImpl.getProjectIdFromPool())),
											false);

		this.outStr.append("\nLook for members of the project ");
		this.outStr.append(p.getName());
		logger.log(
				BasicLevel.DEBUG,
				"Query employee member of the project=" + p.getName());

		Query query = pm.newQuery(Employee.class);

		String filter = "(projects.contains(p))";
		String params = "org.objectweb.speedo.j2eedo.database.Project p";
		String sort = "hiredate ascending, salary descending";

		query.declareParameters(params);
		query.setFilter(filter);
		query.setOrdering(sort);

		Collection col = (Collection) query.execute(p);
		Iterator iter = col.iterator();
		if (!iter.hasNext())
			throw new Exception("Query on employee does not return any row");
		Employee e = null;
		while (iter.hasNext()) {
			e = (Employee) iter.next();
			this.outStr.append("\n\tEmployee name:");
			this.outStr.append(e.getFirstname());
			this.outStr.append(" ");
			this.outStr.append(e.getLastname());
		}

		query.close(col);
	}

	private void queryEmployeeArrayParameters()
	throws JDOException, Exception {
		long idMin =
			DatabaseImpl.getEmployeeIdFromPool();
		long idMax =
			DatabaseImpl.getEmployeeIdFromPool();
		while (idMin == idMax)
			idMax =
				DatabaseImpl.getEmployeeIdFromPool();
		//
		if (idMin > idMax) {
			//  max < min ! inverse boundaries
			long dummy = idMax;
			idMax = idMin;
			idMin = dummy;
		}
		this.outStr.append("\nLook for employees having id between ");
		this.outStr.append(idMin);
		this.outStr.append(" and ");
		this.outStr.append(idMax);

		logger.log(
				BasicLevel.DEBUG,
				"Query employee having id between " + idMin + " and " + idMax);

		Long param[] = new Long[] { new Long(idMin), new Long(idMax)};
		Query query = pm.newQuery(Employee.class);

		String filter = "((empid>=idMin) && (empid<=idMax))";
		String params = "long idMin, long idMax";
		String ordering = "empid descending";

		query.declareParameters(params);
		query.setFilter(filter);
		query.setOrdering(ordering);

		Collection col = (Collection) query.executeWithArray(param);
		Iterator iter = col.iterator();
		if (!iter.hasNext())
			throw new Exception("Query on employee does not return any row");
		Employee e = (Employee) iter.next();
		this.outStr.append("\n\tEmployee name (");
		this.outStr.append(e.getEmpid());
		this.outStr.append("): ");
		this.outStr.append(e.getFirstname());
		this.outStr.append(" ");
		this.outStr.append(e.getLastname());
		if (false && e.getEmpid() != idMax) {
			throw new Exception("Query on employee returns an other first row than expecting");
		}
		while (iter.hasNext()) {
			e = (Employee) iter.next();
			this.outStr.append("\n\tEmployee name (");
			this.outStr.append(e.getEmpid());
			this.outStr.append("): ");
			this.outStr.append(e.getFirstname());
			this.outStr.append(" ");
			this.outStr.append(e.getLastname());
		}
		if (false && e.getEmpid() != idMin)
			throw new Exception("Query on employee returns an other last row than expecting");

		query.close(col);
	}

	private void queryEmployeeByManager() throws JDOException, Exception {
		Employee e =
			(Employee) pm.getObjectById(
					pm.newObjectIdInstance(
							Employee.class,
							Long.toString(
									DatabaseImpl.getEmployeeIdFromPool())),
											false);

		Employee boss = e.getManager();

		this.outStr.append(
		"\nLook for employees having the same manager : ");
		if (null != boss)
			this.outStr.append(boss.getLastname());
		else
			this.outStr.append("NULL");

		logger.log(
				BasicLevel.DEBUG,
				"Query employee having the same manager :"
				+((null==boss)? "null" : boss.getLastname()));

		Query query = pm.newQuery(Employee.class);

		String filter = "(manager==e)";
		String params = "org.objectweb.speedo.j2eedo.database.Employee e";

		query.declareParameters(params);
		query.setFilter(filter);

		Collection col = (Collection) query.execute(boss);
		Iterator iter = col.iterator();
		e = null;
		while (iter.hasNext()) {
			e = (Employee) iter.next();
			this.outStr.append("\n\tEmployee name:");
			this.outStr.append(e.getFirstname());
			this.outStr.append(" ");
			this.outStr.append(e.getLastname());
		}

		query.close(col);
	}
}
