/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Created on 2004-02-18
 * @author fmillevi@yahoo.com
 *
 */
package org.objectweb.speedo.j2eedo.web;

import java.io.IOException;
import java.rmi.RemoteException;

import javax.jdo.JDOException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.speedo.j2eedo.common.PMHolder;
import org.objectweb.speedo.j2eedo.bo.DatabaseImpl;
import org.objectweb.speedo.j2eedo.ejb.StoreServicesLocal;
import org.objectweb.speedo.j2eedo.ejb.StoreServicesRemote;

/**
 * is an extended of the former servlet MainServlet. This class implements the
 * two methods used to execute the resquest through a Session Bean or directly
 * from the {@link org.objectweb.speedo.j2eedo.bo.DatabaseImpl DataBaseImpl} class.</li>
 * </ul>
 *
 * @author fmillevi@yahoo.com
 *
 */
public class ApplicationServlet extends MainServlet {
	/**
	 * The constant <code>PARAMETER_ACTION</code> defines the HTTP
	 * requests paramater <b>"action"</b> use to know
	 * {@link org.objectweb.speedo.j2eedo.bo.DatabaseImpl#actionArray which action} to do.
	 */
	final public static String PARAMETER_ACTION = "action";

	private DatabaseImpl departmentImpl = null;

	/**
	 * @see org.objectweb.speedo.j2eedo.web.MainServlet#executeSessionBean(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.objectweb.speedo.j2eedo.ejb.StoreServicesRemote,
	 *      org.objectweb.speedo.j2eedo.common.PMHolder)
	 */
	protected void executeSessionBean(HttpServletRequest req,
            HttpServletResponse resp, 
            StoreServicesLocal storeServices,
            PMHolder persistenceManagerHolder) throws JDOException,
            RemoteException, Exception, IOException {
        ServletOutputStream out = resp.getOutputStream();
        out.println(storeServices.doAction(
                persistenceManagerHolder, req.getParameter(PARAMETER_ACTION)));
    }

    protected void executeSessionBean(HttpServletRequest req,
            HttpServletResponse resp, 
            StoreServicesRemote storeServices)
            throws JDOException, RemoteException, Exception, IOException {
        ServletOutputStream out = resp.getOutputStream();
        out.println(storeServices.doAction(req.getParameter(PARAMETER_ACTION)));
    }

	/**
	 * @see org.objectweb.speedo.j2eedo.web.MainServlet#executeDirectCall(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse,
	 *      org.objectweb.speedo.j2eedo.common.PMHolder)
	 *      javax.servlet.ServletOutputStream)
	 */
	protected void executeDirectCall(
		HttpServletRequest req,
		HttpServletResponse resp,
		PMHolder persistenceManagerHolder)
		throws JDOException, IOException, Exception {
		ServletOutputStream out = resp.getOutputStream();
		if (departmentImpl == null) {
			departmentImpl = new DatabaseImpl(persistenceManagerHolder);
		}
		out.println(
			departmentImpl.doAction(
				req.getParameter(ApplicationServlet.PARAMETER_ACTION)));
	}
}